-- Civilization 5 Engine Database Schema
-- The following tables are required by the Civ5 engine.
-- Modifying these tables is not recommended.

-- ScannedFiles table used for caching database.
CREATE TABLE ScannedFiles("Path" TEXT NOT NULL PRIMARY KEY, "DateTime" INTEGER NOT NULL);

CREATE TABLE ArtDefine_UnitInfos(	"Type" TEXT NOT NULL PRIMARY KEY,
									"DamageStates" INT,
									"Formation" TEXT);

CREATE TABLE ArtDefine_UnitInfoMemberInfos(	"UnitInfoType" TEXT,
											"UnitMemberInfoType" TEXT,
											"NumMembers" INT,
											FOREIGN KEY("UnitInfoType") REFERENCES ArtDefine_UnitInfos("Type")
											FOREIGN KEY("UnitMemberInfoType") REFERENCES ArtDefine_UnitMemberInfos("Type"));
									

-- Art define tables used by cvUnitLibrary
CREATE TABLE ArtDefine_UnitMemberInfos(	"Type" TEXT NOT NULL PRIMARY KEY, 
										"Scale" FLOAT, 
										"ZOffset" FLOAT,
										"Domain" TEXT,										
										"Model" TEXT NOT NULL,
										"MaterialTypeTag" TEXT,
										"MaterialTypeSoundOverrideTag" TEXT);

CREATE TABLE ArtDefine_UnitMemberCombats(
										"UnitMemberType" TEXT NOT NULL PRIMARY KEY, 
										"EnableActions" TEXT,
										"DisableActions" TEXT,
										"MoveRadius" FLOAT,
										"ShortMoveRadius" FLOAT, 
										"ChargeRadius" FLOAT,
										"AttackRadius" FLOAT,
										"RangedAttackRadius" FLOAT,
										"MoveRate" FLOAT,
										"ShortMoveRate" FLOAT,
										"TurnRateMin" FLOAT,
										"TurnRateMax" FLOAT,
										"TurnFacingRateMin" FLOAT,
										"TurnFacingRateMax" FLOAT,
										"RollRateMin" FLOAT,
										"RollRateMax" FLOAT,
										"PitchRateMin" FLOAT,
										"PitchRateMax" FLOAT,
										"LOSRadiusScale" FLOAT,
										"TargetRadius" FLOAT,
										"TargetHeight" FLOAT,
										"HasShortRangedAttack" INTEGER,
										"HasLongRangedAttack" INTEGER,
										"HasLeftRightAttack" INTEGER,
										"HasStationaryMelee" INTEGER,
										"HasStationaryRangedAttack" INTEGER,
										"HasRefaceAfterCombat" INTEGER,
										"ReformBeforeCombat" INTEGER,
										"HasIndependentWeaponFacing" INTEGER,
										"HasOpponentTracking" INTEGER,
										"HasCollisionAttack" INTEGER,
										"AttackAltitude" FLOAT,
										"AltitudeDecelerationDistance" FLOAT,
										"OnlyTurnInMovementActions" INTEGER,
										"RushAttackFormation" TEXT,
										FOREIGN KEY("UnitMemberType") REFERENCES ArtDefine_UnitMemberInfos("Type"));
										
CREATE TABLE ArtDefine_UnitMemberCombatWeapons(	"UnitMemberType" TEXT NOT NULL,
												"Index" INTEGER NOT NULL,
												"SubIndex" INTEGER NOT NULL,
												"ID" TEXT,
												"VisKillStrengthMin" FLOAT,
												"VisKillStrengthMax" FLOAT,
												"ProjectileSpeed" FLOAT,
												"ProjectileTurnRateMin" FLOAT,
												"ProjectileTurnRateMax" FLOAT,
												"HitEffect" TEXT,
												"HitEffectScale" FLOAT,
												"HitRadius" FLOAT,
												"ProjectileChildEffectScale" FLOAT,
												"AreaDamageDelay" FLOAT,
												"ContinuousFire" INTEGER,
												"WaitForEffectCompletion" INTEGER,
												"TargetGround" INTEGER,
												"IsDropped" INTEGER,
												"WeaponTypeTag" TEXT,
												"WeaponTypeSoundOverrideTag" TEXT,
												PRIMARY KEY("UnitMemberType", "Index" ASC, "SubIndex" ASC));
												
-- Art defines used by cvLandmarkLibrary
CREATE TABLE ArtDefine_LandmarkTypes( "Type" TEXT NOT NULL,
									  "LandmarkType" TEXT NOT NULL,
									  "FriendlyName" TEXT);
									  
CREATE TABLE ArtDefine_Landmarks(	"Era" TEXT DEFAULT "Any",
									"State" TEXT DEFAULT "Any",
									"Scale" FLOAT DEFAULT 1,
									"ImprovementType" TEXT NOT NULL,
									"LayoutHandler" TEXT NOT NULL,
									"ResourceType" TEXT NOT NULL,
									"Model" TEXT,
									"TerrainContour" INTEGER DEFAULT 0,
									"Tech" TEXT,
									FOREIGN KEY("ImprovementType") REFERENCES ArtDefine_LandmarkTypes("Type"),
									FOREIGN KEY("ResourceType") REFERENCES ArtDefine_LandmarkTypes("Type"),
									FOREIGN KEY("LayoutHandler") REFERENCES ArtDefine_LandmarkTypes("Type"));
									
-- Art defines used by Strategic View
CREATE TABLE ArtDefine_StrategicView("StrategicViewType" TEXT,
									 "TileType" TEXT NOT NULL,
									 "Asset" TEXT NOT NULL,
									 PRIMARY KEY("StrategicViewType", "TileType"));	  

-- DLC Definitions
CREATE TABLE DownloadableContent(	"PackageID" TEXT NOT NULL,
									"FriendlyNameKey" TEXT,
									"DescriptionKey" TEXT,
									"Version" INTEGER DEFAULT 1,
									"IsFiraxisContent" INTEGER DEFAULT 0,
									"IsActive" INTEGER DEFAULT 0,
									"IsBaseContentUpgrade" INTEGER DEFAULT 0);

-- Map Script data used by Modding Framework
CREATE TABLE MapScripts('FileName' TEXT PRIMARY KEY COLLATE NOCASE, 'Name' TEXT NOT NULL, 'Description' TEXT, 'IsAdvancedMap' INTEGER DEFAULT 0, 'SupportsSinglePlayer' INTEGER DEFAULT 1, 'SupportsMultiplayer' INTEGER DEFAULT 1, 'DefaultCityStates' INTEGER, 'Hidden' INTEGER DEFAULT 0, 'IconIndex' INTEGER DEFAULT 0, 'IconAtlas' TEXT DEFAULT 'WORLDTYPE_ATLAS');

CREATE TABLE MapScriptOptions('FileName' TEXT NOT NULL COLLATE NOCASE, 'OptionID' TEXT NOT NULL, 'Name' TEXT NOT NULL, 'Description' TEXT, 'ReadOnly' INTEGER DEFAULT 0, 'Hidden' INTEGER DEFAULT 0, 'DefaultValue' INTEGER NOT NULL, 'SortPriority' INTEGER NOT NULL);

CREATE TABLE MapScriptOptionPossibleValues('FileName' TEXT NOT NULL COLLATE NOCASE, 'OptionID' TEXT NOT NULL, 'Name' TEXT NOT NULL, 'Description' TEXT, 'Value' INTEGER NOT NULL, 'SortIndex' INTEGER NOT NULL);

CREATE TABLE MapScriptRequiredDLC('FileName' TEXT NOT NULL COLLATE NOCASE, 'PackageID' TEXT NOT NULL);
-- Application
CREATE TABLE ApplicationInfo( "Version" TEXT NOT NULL );

CREATE TRIGGER OnDeleteMapScript AFTER DELETE ON MapScripts BEGIN 
	DELETE FROM MapScriptOptionPossibleValues WHERE FileName = old.FileName;
	DELETE FROM MapScriptOptions WHERE FileName = old.FileName;
	DELETE FROM MApScriptRequiredDLC WHERE FileName = old.FileName;
END; 