local ShowDebugMessages = Game.IsTutorialLogging();

COMPLETE  = 0; -- Successfully completed this task
DISMISSED = 1; -- Tutorial display was seen and dismissed by player
INVALID   = 2; -- Missed the boat on this task
ACTIVE    = 3; -- This task can be evaluated
INACTIVE  = 4; -- This task cannot currently be evaluated

ADVISOR_SCIENCE  = AdvisorTypes.ADVISOR_SCIENCE;
ADVISOR_ECONOMIC = AdvisorTypes.ADVISOR_ECONOMIC;
ADVISOR_FOREIGN  = AdvisorTypes.ADVISOR_FOREIGN;
ADVISOR_MILITARY = AdvisorTypes.ADVISOR_MILITARY;

-- this is our global info table that will be used in various tutorials
GlobalTutorialInfo = {};

function Tutorial(arg)
	arg.Index = table.count(GlobalTutorialInfo);
	GlobalTutorialInfo[arg.ID] = arg;
end

-- serialized data?
local EverHealedUnit = false;
local bEverCityBombard = false;
local iMaxThingsCityBuilt = 0;
local NumHappinessResourcesConnected = 0;
local strID;

------------------------------------------------------------
------------------------------------------------------------
-- debug functions
function dprint(msg)
	if (ShowDebugMessages) then
		print(msg);
	end
end

--Cache GreatGeneral Unit Type
local ggUnitClass = GameInfo.UnitClasses.UNITCLASS_GREAT_GENERAL;
local ggUnitClassID = nil;
if(ggUnitClass ~= nil) then
	ggUnitClassID = ggUnitClass.ID;
end

------------------------------------------------------------
------------------------------------------------------------
-- utility functions
function GetPlayer ()
	local iPlayerID = Game.GetActivePlayer();
	if (iPlayerID < 0) then
		dprint("Error - player index not correct");
		return nil;
	end

	if (not Players[iPlayerID]:IsHuman()) then
		return nil;
	end;

	return Players[iPlayerID];
end

function IsStrictlyWorker (pUnit)
	if (pUnit:WorkRate() > 0 and not pUnit:IsCombatUnit() and pUnit:GetDomainType() == DomainTypes.DOMAIN_LAND and pUnit:GetSpecialUnitType() == -1) then
		return true;
	else
		return false;
	end	
end

------------------------------------------------------------
------------------------------------------------------------
function BuildFirstCityFirstTurnCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (player:GetNumCities() > 0) then
		return COMPLETE;
	end
	
	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	-- If the unit has already used all its movement this turn
	if (GetBuildFirstCityUnitIndex() == -1) then
		return INACTIVE;
	end
	
	return ACTIVE;
end

function BuildFirstCityCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	--dprint("player:GetNumCities() - " .. player:GetNumCities());
	if (player:GetNumCities() > 0) then
		return COMPLETE;
	end
	
	-- If the unit has already used all its movement this turn
	if (GetBuildFirstCityUnitIndex() == -1) then
		return INACTIVE;
	end
	
	return ACTIVE;
end

function GetBuildFirstCityUnitIndex ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end
	
	for v in player:Units() do
		if (v ~= nil) then
			if (v:IsFound() and v:MovesLeft() > 0) then
				return v:GetID();
			end
		end
	end
    
    return -1;
end

Tutorial { 
	ID = "FIRST_TURN_BUILD_CITY",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = BuildFirstCityFirstTurnCheck,   
	UnitIndexFunction = GetBuildFirstCityUnitIndex, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = 10, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITIES",
	Concept2 = "CONCEPT_FOUNDING_CITIES",
	Concept3 = "CONCEPT_WHERE_TO_FOUND_CITIES",
};

------------------------------------------------------------
------------------------------------------------------------

Tutorial { 
	ID = "LATER_TURNS_BUILD_CITY",    
	ActivateButtonText = true,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = BuildFirstCityCheck,            
	UnitIndexFunction = GetBuildFirstCityUnitIndex, 
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	PesterAgainTurn =  1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITIES",
	Concept2 = "CONCEPT_FOUNDING_CITIES",
	Concept3 = "CONCEPT_WHERE_TO_FOUND_CITIES",	
};

------------------------------------------------------------
------------------------------------------------------------
function MoveUnitsCheck ()
	if (Game.GetGameTurn() > 0) then
		return COMPLETE;
	end
	
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	--if (player:GetNumCities() == 0) then
		--return INACTIVE;
	--end
	
	if (player:HasReadyUnit()) then
		return ACTIVE;
	end

	return COMPLETE;
end

function GetMoveUnitsUnitIndex ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end
	
	for v in player:Units() do
		if (v ~= nil) then
			if (v:IsReadyToMove()) then
				return v:GetID();
			end
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "FIRST_TURN_UNIT_MOVE",      
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN,  
	CheckFunction = MoveUnitsCheck,			        
	UnitIndexFunction = GetMoveUnitsUnitIndex,      
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = 10, 
	TurnsCheck = -1, 
	PesterAgainTurn =  1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_MOVEMENT",
	Concept2 = "CONCEPT_FOG_OF_WAR",
	Concept3 = "CONCEPT_UNITS",
};

------------------------------------------------------------
------------------------------------------------------------
function LostAllMilitaryUnitsCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	if (Game.GetGameTurn() > 100) then
		return COMPLETE;
	end

	if (Game.GetGameTurn() < 25) then
		return INACTIVE;
	end
	
	--if (player:GetNumMilitaryUnits() > 0) then
	--	return INACTIVE;
	--end
	
	-- if there is a unit in friendly territory, we're good for now
	for v in player:Units() do
		local pPlot = v:GetPlot();
		if (v:IsCombatUnit() and pPlot:GetOwner() == v:GetOwner()) then
			return INACTIVE;
		end
	end
	
	-- see if any city has a military unit queued next
	for v in player:Cities() do
		local UnitType = v:GetProductionUnit();
		if (UnitType >= 0 and GameInfo.Units[UnitType].Combat > 0) then
			dprint("City has military unit queued");
			return INACTIVE;
		end
	end
	
	-- see if any cities are done with production
	for v in player:Cities() do
		if (v:GetProduction() == 0) then
			return ACTIVE;	
		end
	end
			
	return INACTIVE;
end

Tutorial { 
	ID = "LOST_ALL_MILITARY",         
	ActivateButtonText = false, 
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = LostAllMilitaryUnitsCheck,	    
	UnitIndexFunction = nil,					    
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn =  -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITY_CONSTRUCTING_UNITS",
	Concept2 = "CONCEPT_UNITS_COMBAT_UNITS",
	Concept3 = "CONCEPT_VICTORY_DEFEAT",	
};

------------------------------------------------------------
------------------------------------------------------------
function WorkerOutOfTerritoryCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (Game.GetGameTurn() > 100) then
		return COMPLETE;
	end
	
	if (player:GetNumCities() > 1) then
		return COMPLETE;
	end
	
	for v in player:Units() do
		if (IsStrictlyWorker(v)) then
			local plot = v:GetPlot();
			if (plot:GetOwner() ~= Game.GetActivePlayer() and player:GetPlotDanger(plot) > 0) then
				return ACTIVE;				
			end	
		end
	end
	
	return INACTIVE;
end

function GetAWOLWorkerIndex ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	for v in player:Units() do
		if (IsStrictlyWorker(v)) then
			local plot = v:GetPlot();
			if (plot:GetOwner() ~= Game.GetActivePlayer() and player:GetPlotDanger(plot) > 0) then
				return v:GetID();
			end	
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "WORKERS_PROTECT",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = WorkerOutOfTerritoryCheck,      
	UnitIndexFunction = GetAWOLWorkerIndex,		    
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  1, 
	PesterAgainTurn =  3,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_UNITS_COMBAT",
	Concept2 = "CONCEPT_WORKERS",
	Concept3 = "CONCEPT_BARBARIANS_CAPTURING_CIVILIANS",	
};

------------------------------------------------------------
------------------------------------------------------------
function GoodyHutCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end
	
	if (player:GetEverPoppedGoody()) then
		--dprint("Player has popped goody");
		return COMPLETE;
	end	

	-- if there is a unit in route, mark the step as inactive for now?
	if (player:GetAnyUnitHasOrderToGoody()) then
		--dprint("Player has a unit directed to get the goody");
		return INACTIVE;
	end
  
	if (player:IsAnyGoodyPlotAccessible()) then
		return ACTIVE;
	end
  
	return INACTIVE;
end

function GetGoodyHutPlot ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return nil;	
	end

	return player:GetClosestGoodyPlot();	
end

Tutorial { 
	ID = "GO_TO_GOODY_HUT",           
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN,  
	CheckFunction = GoodyHutCheck,				    
	UnitIndexFunction = nil,						
	PlotFunction = GetGoodyHutPlot,	      
	MinTurn =  1,  
	TurnsCheck = -1, 
	PesterAgainTurn = 5,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_ANCIENT_RUINS",
	Concept2 = "CONCEPT_ANCIENT_RUINS_BENEFITS",
	Concept3 = "CONCEPT_MOVEMENT",	
};

------------------------------------------------------------
------------------------------------------------------------
function WorkerIdleCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	if (player:IsAnyPlotImproved()) then
		return COMPLETE;	
	end
	
	local AnyWorker = false;
	for v in player:Units() do
		if (IsStrictlyWorker(v) and v:IsSelected()) then
			local plot = v:GetPlot();
			if (player:GetPlotDanger(plot) == 0) then
				AnyWorker = true;
				for pBuildInfo in GameInfo.Builds() do
					local iTurnsLeft = plot:GetBuildTurnsLeft(pBuildInfo.ID, 0, 0);
					dprint("Num Turns Left: " .. iTurnsLeft);
					if (iTurnsLeft < 4000 and iTurnsLeft > 0) then
						return INACTIVE;
					end			
				end
			end
		end
	end
	
	if (AnyWorker) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

function GetIdleWorkerIndex ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	local iWorkerID = -1;
	local bFoundSelected = false;
	for v in player:Units() do
		if (IsStrictlyWorker(v) and not bFoundSelected) then
			iWorkerID = v:GetID();
			if (v:IsSelected()) then
				bFoundSelected = true;
			end
		end
	end
	
	return iWorkerID;
end

Tutorial { 
	ID = "WORKERS_NEED_WORK",			
	ActivateButtonText = true,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = WorkerIdleCheck,				
	UnitIndexFunction = GetIdleWorkerIndex,		    
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  3, 
	PesterAgainTurn =  3,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITY_IMPROVINGLAND",
	Concept2 = "CONCEPT_WORKERS_IMPROVEMENTS",
	Concept3 = "CONCEPT_WORKERS_ROADS",	
};

------------------------------------------------------------
------------------------------------------------------------
function BuildMoreMilitaryCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end	
	
	if (Game.GetGameTurn() > 50 or player:GetNumMilitaryUnits() >= 2) then
		return COMPLETE;
	end
	
	-- see if any city has a military unit queued next
	for v in player:Cities() do
		local UnitType = v:GetProductionUnit();
		if (UnitType >= 0 and GameInfo.Units[UnitType].Combat > 0) then
			dprint("City has military unit queued");
			return INACTIVE;
		end
	end
	
	-- see if any cities are done with production
	for v in player:Cities() do
		if (v:GetProduction() == 0) then
			if (player:GetNumMilitaryUnits() <= player:GetNumCities()) then
				return ACTIVE;
			end		
		end
	end
		
	return INACTIVE;
end

Tutorial { 
	ID = "BUILD_MORE_MILITARY",	    
	ActivateButtonText = false,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = BuildMoreMilitaryCheck,		    
	UnitIndexFunction = nil,						
	PlotFunction = nil,				      
	MinTurn = 15, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITY_CONSTRUCTING_UNITS",
	Concept2 = "CONCEPT_COMBAT",
	Concept3 = "CONCEPT_VICTORY_VICTORY_DOMINATION",	
};

------------------------------------------------------------
------------------------------------------------------------
function UnitHealCheck ()
	if (EverHealedUnit) then
		return COMPLETE;
	end

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	local ReturnValue = INACTIVE;
	
	for v in player:Units() do
		if (v:GetActivityType() == ActivityTypes.ACTIVITY_HEAL) then
			ReturnValue = COMPLETE;
			break;
		end

	
		local plot = v:GetPlot();
		if (v:IsHurt() and v:CanHeal(plot) and v:MovesLeft() > 0 and v:IsSelected()) then
			if (v:GetCurrHitPoints() <= (v:GetMaxHitPoints() / 3)) then		
				--dprint("Found a hurt unit with moves left, active");
				ReturnValue = ACTIVE;
				break;
			end
		end
	end
	
	return ReturnValue;
end

function GetUnitHealUnitIndex ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	local iUnitID = -1;
	local bFoundSelected = false;
	for v in player:Units() do
		if (v ~= nil and not bFoundSelected) then
			local plot = v:GetPlot();
			if (v:IsHurt() and v:CanHeal(plot) and v:MovesLeft() > 0) then
				iUnitID = v:GetID();
				if (v:IsSelected()) then
					bFoundSelected = true;
				end
			end
		end
	end
    
    return iUnitID;
end

Tutorial { 
	ID = "UNIT_HEAL",					
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = UnitHealCheck,			        
	UnitIndexFunction = GetUnitHealUnitIndex,		
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn =  5,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_COMBAT_FORTIFY_UNTIL_HEALED",
	Concept2 = "CONCEPT_COMBAT_UNIT_GROUND_HEALING_DAMAGE",
	Concept3 = "CONCEPT_COMBAT_UNIT_STATS_HIT_POINTS",	
};

------------------------------------------------------------
------------------------------------------------------------
function TrainWorkerCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	-- make sure the player has a city before we direct them to train a worker
	if (player:GetNumCities() == 0) then
		return INACTIVE;
	end
	
	if (Game.GetGameTurn() >= 100) then
		return COMPLETE;
	end
	
	for v in player:Units() do
		if (IsStrictlyWorker(v)) then
			return COMPLETE;
		end
	end
	
	-- see if any city is building a worker
	for v in player:Cities() do
		--dprint("Cycling through cities " .. i);
		if (v ~= nil) then
			if (v:GetFirstUnitOrder(GameInfo.Units.UNIT_WORKER.ID) ~= -1) then
				--dprint("Found a builder");
				return INACTIVE;
			end
			--dprint("Still looking");
		end
	end
		
	-- see if any cities are done with production
	for v in player:Cities() do
		if (v:GetProduction() == 0) then
			return ACTIVE;	
		end
	end	
		
	return INACTIVE;
end

Tutorial { 
	ID = "WORKERS_TRAIN",				
	ActivateButtonText = false,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = TrainWorkerCheck,			    
	UnitIndexFunction = nil,						
	PlotFunction = nil,				      
	MinTurn = 30, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = 10,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_WORKERS",	
	Concept2 = "CONCEPT_CITY_IMPROVINGLAND",
	Concept3 = "CONCEPT_RESOURCES",
};

------------------------------------------------------------
------------------------------------------------------------
function NotMovingUnitsCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	-- make sure the player has a city before we bother them
	--if (player:GetNumCities() == 0) then
		--return INACTIVE;
	--end
	
	
	--dprint("Checking military units");
	if (player:GetNumMilitaryUnits() ~= 1) then
		return INACTIVE;
	end

	local mostRecentLastMoveTurn = 0;

	--dprint("Game.GetGameTurn() = " .. Game.GetGameTurn());

	for v in player:Units() do
		--dprint("v:GetLastMoveTurn() " .. v:GetLastMoveTurn());
		--dprint("v:GetGameTurnCreated() " .. v:GetGameTurnCreated());
		local lastActivation = v:GetLastMoveTurn();
		if (v:GetGameTurnCreated() > lastActivation) then
			lastActivation = v:GetGameTurnCreated();
		end
		
		if (lastActivation > mostRecentLastMoveTurn) then
			mostRecentLastMoveTurn = lastActivation;
		end		
		
		--dprint("v:mostRecentLastMoveTurn" .. mostRecentLastMoveTurn);
	end
	
	
	--dprint("Game.GetGameTurn() - mostRecentLastMoveTurn" .. Game.GetGameTurn() - mostRecentLastMoveTurn);
	if (Game.GetGameTurn() - mostRecentLastMoveTurn >= 3) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

function GetNotMovingUnitsUnitIndex ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return;	
	end

	for v in player:Units() do
		return v:GetID();
	end
end

Tutorial { 
	ID = "LATER_TURN_UNIT_MOVE",      
	ActivateButtonText = true,  
	Advisor = ADVISOR_FOREIGN,  
	CheckFunction = NotMovingUnitsCheck,			
	UnitIndexFunction = GetNotMovingUnitsUnitIndex, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MaxTurn = 15,
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn =  2,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_MOVEMENT",
	Concept2 = "CONCEPT_FOG_OF_WAR",
	Concept3 = "CONCEPT_BARBARIANS_AND_ANCIENT_RUINS",	
};

------------------------------------------------------------
------------------------------------------------------------
function FirstConstructionCompleteCheck ()
	dprint("begin FirstConstructionCompleteCheck");
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		dprint("end FirstConstructionCompleteCheck - INACTIVE");
		return INACTIVE;	
	end
	
	-- see if any cities are done with production
	local result = INACTIVE;
	
	if (iMaxThingsCityBuilt == 0) then
		for v in player:Cities() do
			dprint("Cycling through cities");
			if (v:GetNumThingsProduced() > 1) then
				result = COMPLETE;
			elseif (v:GetNumThingsProduced() == 1 and result == INACTIVE) then
				result = ACTIVE;
			end
		end	
	end
		
	--dprint("end FirstConstructionCompleteCheck - INACTIVE");
	return result;
end

Tutorial { 
	ID = "CITY_FIRST_CONSTRUCTION",	
	ActivateButtonText = false,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = FirstConstructionCompleteCheck, 
	UnitIndexFunction = nil,                        
	PlotFunction = nil,                   
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_CITY_CONSTRUCTION",
	Concept2 = "CONCEPT_CITY_BUILD",
	Concept3 = "CONCEPT_CITY_CHANGE_BUILD",	
};

------------------------------------------------------------
------------------------------------------------------------
function WorkerBuiltCheck ()
	dprint("WorkerBuildCheck begin");

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		dprint("end FirstConstructionCompleteCheck - INACTIVE");
		return INACTIVE;	
	end
			
	if (player:IsAnyPlotImproved()) then
		dprint("WorkerBuildCheck end - COMPLETE");
		return COMPLETE;
	end
	
	for v in player:Units() do
		if (IsStrictlyWorker(v) and v:IsSelected()) then
			return ACTIVE;
		end
	end

	return INACTIVE;
end

function GetWorkerIndex ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	for v in player:Units() do
		if (IsStrictlyWorker(v)) then
			return v:GetID();
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "WORKERS_INTRO",				
	ActivateButtonText = true,  
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = WorkerBuiltCheck,               
	UnitIndexFunction = GetWorkerIndex,			    
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_WORKERS",
	Concept2 = "CONCEPT_CITY_IMPROVINGLAND",
	Concept3 = "CONCEPT_FOOD_AND_CITY_GROWTH_GETTING_MORE_FOOD",	
};

------------------------------------------------------------
------------------------------------------------------------
function PeopleUnhappyCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return;	
	end

	if (not player:IsEmpireUnhappy()) then
		return INACTIVE;
	end

	local bHasWorker = false;
	for v in player:Units() do
		if (IsStrictlyWorker(v)) then
			bHasWorker = true;
		end
	end
	
	if (not bHasWorker) then
		return INACTIVE;
	end

	local plot = player:GetUnimprovedAvailableLuxuryResource();
	if (plot == nil) then
		return INACTIVE;
	end
		
	return ACTIVE;
end

function GetLuxuryResourcePlot ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return;	
	end
	
	return player:GetUnimprovedAvailableLuxuryResource();
end

Tutorial { 
	ID = "HAPPINESS_RESOURCE",		
	ActivateButtonText = true,  
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = PeopleUnhappyCheck,			    
	UnitIndexFunction = nil,						
	PlotFunction = GetLuxuryResourcePlot, 
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = 20, 
	TutorialLevel = 1,
	Concept1 = "CONCEPT_HAPPINESS_UNHAHPPY",
	Concept2 = "CONCEPT_RESOURCES_LUXURY",
	Concept3 = "CONCEPT_FOOD_AND_CITY_GROWTH_UNHAPPINESS",	
};

------------------------------------------------------------
------------------------------------------------------------
function PeopleVeryUnhappyCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (player:IsEmpireVeryUnhappy()) then
		return ACTIVE;
	end

	return INACTIVE;
end

Tutorial { 
	ID = "VERY_UNHAPPY",		
	ActivateButtonText = false,  
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = PeopleVeryUnhappyCheck,			    
	UnitIndexFunction = nil,				
	PlotFunction = nil, 
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = 20, 
	TutorialLevel = 1,
	Concept1 = "CONCEPT_HAPPINESS_VERY_UNHAPPY",
	Concept2 = "CONCEPT_RESOURCES_LUXURY",
	Concept3 = "CONCEPT_FOOD_AND_CITY_GROWTH_UNHAPPINESS",	
};

------------------------------------------------------------
------------------------------------------------------------
function BuildSecondCityCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	if (player:GetNumCities() > 1) then
		return COMPLETE;
	end
	
	if (Game.GetGameTurn() >= 150) then
		return COMPLETE;
	end
	
	if (Game.IsOption( GameOptionTypes.GAMEOPTION_ONE_CITY_CHALLENGE )) then
		return INACTIVE;
	end
	
	-- Do we already have a settler on the map?
	for v in player:Units() do
		local UnitType = v:GetUnitType();
		if (UnitType >= 0 and GameInfo.Units[UnitType].Found) then
			return INACTIVE;
		end
	end
	
	-- see if any city is building a builder
	for v in player:Cities() do
		local UnitType = v:GetProductionUnit();
		if (UnitType >= 0 and GameInfo.Units[UnitType].Found) then
			return INACTIVE;
		end
	end
	
	-- see if any cities are done with production
	for v in player:Cities() do
		if (v:GetProduction() == 0) then
			return ACTIVE;	
		end
	end	
	
	return INACTIVE;
end

Tutorial { 
	ID = "BUILD_NEW_CITY",            
	ActivateButtonText = false,
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = BuildSecondCityCheck,		    
	UnitIndexFunction = nil,						
	PlotFunction = nil,				      
	MinTurn = 60, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn =  50,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_FOUNDING_CITIES",
	Concept2 = "CONCEPT_WHERE_TO_FOUND_CITIES",
	Concept3 = "CONCEPT_FOOD_AND_CITY_GROWTH_BEST_FOOD_TILES",	
};

------------------------------------------------------------
------------------------------------------------------------
local iLastTurnNumMinorsMet = 0;
if (GetPlayer()) then
	iLastTurnNumMinorsMet = GetPlayer():GetNumMinorCivsMet();
end

function MetFirstCityStateCheck ()
	dprint("MetFirstCityStateCheck begin");
	dprint("iLastTurnNumMinorsMet: " .. iLastTurnNumMinorsMet);
	dprint("player:GetNumMinorCivsMet(): " .. GetPlayer():GetNumMinorCivsMet());
	dprint("MetFirstCityStateCheck end");

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (iLastTurnNumMinorsMet > 0) then
		return COMPLETE;
	end

	if (player:GetNumMinorCivsMet() == 0) then
		return INACTIVE;
	end

	return ACTIVE;
end

function GetCityStatePlot ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return nil;	
	end
	
	local pTeam = Teams[player:GetTeam()];

	local pMinorCiv = nil;

	-- Loop through all the Majors the active player knows
	for iPlayerLoop = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local pOtherPlayer = Players[iPlayerLoop];
		local iOtherTeam = pOtherPlayer:GetTeam();
		-- Valid player? - Can't be us, and has to be alive
		if (player:GetTeam() ~= iOtherTeam and pOtherPlayer:IsAlive()) then			
			-- Met this player?
			if (pTeam:IsHasMet(iOtherTeam)) then
				pMinorCiv = pOtherPlayer;
			end
		end
	end
	
	-- return capital plot if we can see it
	local pCapitalCity = pMinorCiv:GetCapitalCity();
	if (pCapitalCity) then
		local pCapitalCityPlot = pCapitalCity:Plot();
		if (pCapitalCityPlot:IsRevealed(player:GetTeam())) then
			return pCapitalCityPlot;
		end
	end
	
	-- return unit plot if we can see it
	for i, v in pMinorCiv:Units() do
		local pUnitPlot = v:GetPlot();
		if (pUnitPlot:IsRevealed(player:GetTeam())) then
			return pUnitPlot;
		end
	end

	-- return tile if we can see it
	return pMinorCiv:GetPlayerVisiblePlot(player);
end

Tutorial { 
	ID = "CITY_STATES_INTRO",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_FOREIGN,  
	CheckFunction = MetFirstCityStateCheck,		    
	UnitIndexFunction = nil,						
	PlotFunction = GetCityStatePlot,	  
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_CITY_STATE",
	Concept2 = "CONCEPT_CITY_STATE_COMMUNICATION",
	Concept3 = "CONCEPT_DIPLOMACY_CITY_STATE",	
};

------------------------------------------------------------
------------------------------------------------------------
function CityStateQuestCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	local pTeam = Teams[player:GetTeam()];

	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do			
		pOtherPlayer = Players[iPlayerLoop];
		iOtherTeam = pOtherPlayer:GetTeam();
		
		if (pOtherPlayer:IsMinorCiv() and pOtherPlayer:IsAlive() and not pTeam:IsAtWar(iOtherTeam)) then
			if (pTeam:IsHasMet(iOtherTeam)) then
				local iQuest = pOtherPlayer:GetActiveQuestForPlayer(player:GetID());
				if (iQuest >= 0) then
					return ACTIVE;
				end
				
				if (pOtherPlayer:GetTurnsSinceThreatenedByBarbarians() >= 0) then
					return ACTIVE;
				end
				
				if (pOtherPlayer:IsMinorWarQuestWithMajorActive()) then
					return ACTIVE;
				end
			end
		end
	end		

	return INACTIVE;
end

function GetCityStateQuestPlot ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return nil;	
	end
	
	local pTeam = Teams[player:GetTeam()];

	local iResultPlayer = -1
	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do		
		pOtherPlayer = Players[iPlayerLoop];
		iOtherTeam = pOtherPlayer:GetTeam();
		
		if (pOtherPlayer:IsMinorCiv() and pOtherPlayer:IsAlive() and not pTeam:IsAtWar(iOtherTeam)) then
			if (pTeam:IsHasMet(iOtherTeam)) then
				local iQuest = pOtherPlayer:GetActiveQuestForPlayer(player:GetID());
				if (iQuest >= 0) then
					iResultPlayer = iPlayerLoop;
				end
				
				if (pOtherPlayer:GetTurnsSinceThreatenedByBarbarians() >= 0) then
					iResultPlayer = iPlayerLoop;
				end
				
				if (pOtherPlayer:IsMinorWarQuestWithMajorActive()) then
					iResultPlayer = iPlayerLoop;
				end
			end
		end
		
		if (iResultPlayer ~= -1) then
			iPlayerLoop = GameDefines.MAX_CIV_PLAYERS;
		end
	end		
	
	if (iResultPlayer == -1) then
		return nil;
	end
	
	local pTeam = Teams[player:GetTeam()];

	local pMinorCiv = Players[iResultPlayer];
	
	-- return capital plot if we can see it
	local pCapitalCity = pMinorCiv:GetCapitalCity();
	if (pCapitalCity) then
		local pCapitalCityPlot = pCapitalCity:Plot();
		if (pCapitalCityPlot:IsRevealed(player:GetTeam())) then
			return pCapitalCityPlot;
		end
	end
	
	-- return unit plot if we can see it
	for i, v in pMinorCiv:Units() do
		local pUnitPlot = v:GetPlot();
		if (pUnitPlot:IsRevealed(player:GetTeam())) then
			return pUnitPlot;
		end
	end

	-- return tile if we can see it
	return pMinorCiv:GetPlayerVisiblePlot(player);
end

Tutorial { 
	ID = "CITY_STATES_QUEST",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_FOREIGN,
	CheckFunction = CityStateQuestCheck,		    
	UnitIndexFunction = nil,						
	PlotFunction = GetCityStateQuestPlot,
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_CITY_STATE_MISSIONS",
	Concept2 = "CONCEPT_CITY_STATE",
	Concept3 = "CONCEPT_CITY_STATE_INFLUENCE",	
};

------------------------------------------------------------
------------------------------------------------------------
function FirstAttackCheck ()

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (Game:IsEverAttackedTutorial()) then
		return COMPLETE;
	end

	for v in player:Units() do
		if (v:IsCanAttack() and v:IsSelected() and not v:IsBusy()) then
			if (v:IsEnemyInMovementRange(false, false)) then
				return ACTIVE;
			end
		end
	end

	return INACTIVE;
end

function GetFirstAttackEnemyIndex ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	local iUnitID = -1;
	local bFoundSelected = false;

	for v in player:Units() do
		if (v:IsCanAttack()) then
			if (v:IsEnemyInMovementRange(false, false)) then
				return v:GetID();
			end
		end
	end

	return -1;	
end

Tutorial { 
	ID = "COMBAT_INTRO",				
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = FirstAttackCheck,               
	UnitIndexFunction = GetFirstAttackEnemyIndex,   
	PlotFunction = nil,                   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_COMBAT",
	Concept2 = "CONCEPT_COMBAT_DECLARE_WAR",
	Concept3 = "CONCEPT_COMBAT_END_WAR",	
};

------------------------------------------------------------
------------------------------------------------------------
function TooManyWorkersCheck ()
	dprint("TooManyWorkersCheck begin");

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		dprint("TooManyWorkersCheck end - INACTIVE");
		return INACTIVE;	
	end
	
	if (player:GetNumCities() > 3) then
		dprint("TooManyWorkersCheck end - COMPLETE");
		return COMPLETE;
	end
	
	if (Game.GetGameTurn() >= 100) then
		dprint("TooManyWorkersCheck end - COMPLETE");
		return COMPLETE;
	end

	local iNumWorkers = 0;

	for v in player:Units() do
		if (IsStrictlyWorker(v)) then
			iNumWorkers = iNumWorkers + 1;
		end
	end

	if (iNumWorkers > player:GetNumCities()) then
		dprint("TooManyWorkersCheck end - ACTIVE");
		return ACTIVE;
	end
	
	-- This should check for captured workers. Only bring this up when the player builds a worker?
	
	dprint("TooManyWorkersCheck end - INACTIVE");
	return INACTIVE;
end

Tutorial { 
	ID = "WORKERS_STOP_TRAINING",		
	ActivateButtonText = false, 
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = TooManyWorkersCheck,            
	UnitIndexFunction = nil,                        
	PlotFunction = nil,                   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1 ,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_ADVISORS",
	Concept2 = "",
	Concept3 = "",	
};

------------------------------------------------------------
------------------------------------------------------------
function UnguardedSettlerCheck ()
	dprint("UnguardedSettlerCheck begin");

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		dprint("UnguardedSettlerCheck end - INACTIVE");
		return INACTIVE;	
	end

	if (player:GetNumCities() == 0) then
		dprint("UnguardedSettlerCheck end - INACTIVE");
		return INACTIVE;
	end
	
	local bUnguardedSettler = false;
	for v in player:Units() do
		if (v:IsFound()) then
			local pPlot = v:GetPlot();
			if (pPlot and pPlot:GetOwner() ~= v:GetOwner() and pPlot:GetUnitPower() <= 0) then
				return ACTIVE;
			end
		end
	end
	
	return INACTIVE;
end

function GetUnguardedSettlerIndex()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	for v in player:Units() do
		if (v:IsFound()) then
			local pPlot = v:GetPlot();
			if (pPlot and pPlot:GetOwner() ~= v:GetOwner() and pPlot:GetUnitPower() <= 0) then
				return v:GetID();
			end
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "SETTLER_PROTECT",		    
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = UnguardedSettlerCheck,          
	UnitIndexFunction = GetUnguardedSettlerIndex,   
	PlotFunction = nil,                   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_COMBAT_WHICH_UNITS_CAN_FIGHT",
	Concept2 = "CONCEPT_MOVEMENT",
	Concept3 = "CONCEPT_UNITS_NON_COMBAT_UNITS",	
};

------------------------------------------------------------
------------------------------------------------------------
function BesiegedCityCheck ()
	dprint("BesiegedCityCheck begin");

	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		dprint("BesiegedCityCheck end - INACTIVE");
		return INACTIVE;	
	end

	for v in player:Cities() do
		local pPlot = v:Plot();
		if (pPlot) then
			local iRange = 1;
			local iX = pPlot:GetX();
			local iY = pPlot:GetY();

			for iDX = -iRange, iRange, 1 do
				for iDY = -iRange, iRange, 1 do
					local pTargetPlot = Map.GetPlotXY(iX, iY, iDX, iDY);
					if (not pTargetPlot:IsWater() and pTargetPlot:IsVisibleEnemyUnit(player:GetID())) then
						return ACTIVE;
					end
				end
			end
		end
	end	
		
	return INACTIVE;
end

function GetBesiegedCityPlot ()
	local player = GetPlayer();
	if (player == nil) then
		return nil;	
	end

	for v in player:Cities() do
		local pPlot = v:Plot();
		if (pPlot) then
			local iRange = 1;
			local iX = pPlot:GetX();
			local iY = pPlot:GetY();

			for iDX = -iRange, iRange, 1 do
				for iDY = -iRange, iRange, 1 do
					local pTargetPlot = Map.GetPlotXY(iX, iY, iDX, iDY);
					if (not pTargetPlot:IsWater() and pTargetPlot:IsVisibleEnemyUnit(player:GetID())) then
						return pPlot;
					end
				end
			end
		end
	end		
	
	return nil;
end

Tutorial { 
	ID = "CITY_UNDER_ATTACK",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = BesiegedCityCheck,              
	UnitIndexFunction = nil,                        
	PlotFunction = GetBesiegedCityPlot,   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,
	Concept1 = "CONCEPT_CITY_COMBAT",
	Concept2 = "CONCEPT_CITY_DEFENDING",
	Concept3 = "CONCEPT_CITY_CONSTRUCTING_UNITS",	
};

------------------------------------------------------------
------------------------------------------------------------
function FirstSettlerCheck()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:IsFound() and  v:MovesLeft() > 0 and v:IsSelected()) then
			return ACTIVE;
		end
	end
	
	return INACTIVE;
end

function GetFirstSettler()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	for v in player:Units() do
		if (v:IsFound()) then
			return v:GetID();
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "SETTLER_INTRO",				
	ActivateButtonText = true,  
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = FirstSettlerCheck,              
	UnitIndexFunction = GetFirstSettler,            
	PlotFunction = nil,                   
	MinTurn = 10, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	Concept1 = "CONCEPT_FOUNDING_CITIES",
	Concept2 = "CONCEPT_WHERE_TO_FOUND_CITIES",
	Concept3 = "CONCEPT_COMBAT_WHICH_UNITS_CAN_FIGHT",	
};
------------------------------------------------------------
------------------------------------------------------------
function OneUnitPerTileCheck ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

    local move_denominator = GameDefines["MOVE_DENOMINATOR"];

	for v in player:Units() do
		if (v:MovesLeft() > 0 and v:IsSelected()) then
			local bCombatUnit = v:IsCombatUnit();
			local pFirstUnitPlot = v:GetPlot();
			local iMovesLeft = v:MovesLeft() / move_denominator;					
			for w in player:Units() do
				local bCombatSecondUnit = w:IsCombatUnit();
				if (v ~= w and bCombatUnit == bCombatSecondUnit) then
					local pSecondUnitPlot = w:GetPlot();					
					if (Map.PlotDistance(pFirstUnitPlot:GetX(), pFirstUnitPlot:GetY(), pSecondUnitPlot:GetX(), pSecondUnitPlot:GetY()) <= iMovesLeft ) then
						return ACTIVE;
					end
				end
			end		
		end
	end

	return INACTIVE;	
end

Tutorial { 
	ID =  "ONE_UNIT_PER_TILE",         
	ActivateButtonText = false, 
	Advisor = ADVISOR_SCIENCE,  
	CheckFunction = OneUnitPerTileCheck,            
	UnitIndexFunction = nil,                        
	PlotFunction = nil,                   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_MOVEMENT",
	Concept2 = "CONCEPT_MOVEMENT_STACKING_LIMITS",
	Concept3 = "CONCEPT_UNITS",
};

------------------------------------------------------------
------------------------------------------------------------
function MilitaryUpkeepCheck ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	if (player:GetNumCities() <= 0) then
		return INACTIVE;
	end

	if (player:GetNumUnitsOutOfSupply() > 0) then
		return ACTIVE;
	end

	return INACTIVE;
end

Tutorial { 
	ID = "MILITARY_UPKEEP",           
	ActivateButtonText = false, 
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = MilitaryUpkeepCheck,            
	UnitIndexFunction = nil,                        
	PlotFunction = nil,                   
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_GOLD_UNIT_BUILDING_MAINTENANCE",
	Concept2 = "CONCEPT_ADVISORS",
	Concept3 = "",

};

------------------------------------------------------------
------------------------------------------------------------

function FirstRangedUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:IsRanged() and v:GetDomainType() == DomainTypes.DOMAIN_LAND and v:IsSelected()) then
			return ACTIVE;
		end
	end

	return INACTIVE;
end

function GetRangedUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	for v in player:Units() do
		if (v:IsRanged() and v:GetDomainType() == DomainTypes.DOMAIN_LAND) then
			return v:GetID();
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "RANGED_UNIT",				
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = FirstRangedUnit,				
	UnitIndexFunction = GetRangedUnit,				
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_ATTACKING_RANGED",
	Concept2 = "CONCEPT_COMBAT_RANGED_COMBAT",
	Concept3 = "CONCEPT_COMBAT_RANGED_COMBAT_RANGE",

};

------------------------------------------------------------
------------------------------------------------------------

function UnitUpgrade ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:CanUpgradeRightNow() and not v:IsBusy()) then
			return ACTIVE;
		end
	end

	return INACTIVE;
end

function GetUpgradableUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	for v in player:Units() do
		if (v:CanUpgradeRightNow()) then
			return v:GetID();
		end
	end

	return -1;
end

Tutorial { 
	ID = "UPGRADING_UNITS",			
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = UnitUpgrade,					
	UnitIndexFunction = GetUpgradableUnit,          
	PlotFunction = nil,					  
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_GOLD_UPGRADE_UNITS",
	Concept2 = "CONCEPT_UNITS_COMBAT_UNITS",
	Concept3 = "",

};		   

------------------------------------------------------------
------------------------------------------------------------

function EmbarkCheck ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end	

	local pTeam = Teams[player:GetTeam()];
	if (pTeam:CanEmbark()) then
		return ACTIVE;
	end

	return INACTIVE;
end

Tutorial { 
	ID = "EMBARKING_UNITS",			
	ActivateButtonText = false,  
	Advisor = ADVISOR_SCIENCE,  
	CheckFunction = EmbarkCheck,					
	UnitIndexFunction = nil,						
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_MOVEMENT_EMBARKING",
	Concept2 = "CONCEPT_TECHNOLOGY",
	Concept3 = "",

};

------------------------------------------------------------
------------------------------------------------------------

function FortifiedCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:IsCombatUnit() and not v:IsBusy()) then
			if (v:IsEnemyInMovementRange(true, false)) then
				return ACTIVE;
			end
		end
	end

	return INACTIVE;
end

function GetUnitNearFortifiedUnit ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	for v in player:Units() do
		if (v:IsCombatUnit()) then
			if (v:IsEnemyInMovementRange(true, false)) then
				return v:GetID();
			end
		end
	end

	return -1;	
end

Tutorial { 
	ID = "ATTACKING_FORTIFIED_UNITS",	
	ActivateButtonText = true,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = FortifiedCheck,					
	UnitIndexFunction = GetUnitNearFortifiedUnit,	
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	Concept1 = "CONCEPT_COMBAT_BONUSES_FORTIFICATION",
	Concept2 = "CONCEPT_COMBAT_FORTIFICATION_EXPLANATION",
	Concept3 = "CONCEPT_COMBAT_FORTIFICATION_WHICH_UNITS_CAN_FORTIFY",

};

------------------------------------------------------------
------------------------------------------------------------

function CityCombatCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:IsCombatUnit()) then
			if (v:IsEnemyInMovementRange(false, true)) then
				return ACTIVE;
			end
		end
	end

	return INACTIVE;
end

function GetNearEnemyCityUnit ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	for v in player:Units() do
		if (v:IsCombatUnit()) then
			if (v:IsEnemyInMovementRange(false, true)) then
				return v:GetID();
			end
		end
	end

	return -1;	
end

Tutorial { 
	ID =  "ATTACKING_CITY",			
	ActivateButtonText = true,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = CityCombatCheck,				
	UnitIndexFunction = GetNearEnemyCityUnit,		
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_COMBAT",
	Concept2 = "CONCEPT_CITY_ATTACKING_MELEE",
	Concept3 = "CONCEPT_CITY_CONQUERING",

};

------------------------------------------------------------
------------------------------------------------------------

function WereRichCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (player:GetGold() >= 400) then
		return ACTIVE;
	end

	return INACTIVE;
end

Tutorial { 
	ID = "LOTS_OF_MONEY",			    
	ActivateButtonText = false,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = WereRichCheck,					
	UnitIndexFunction = nil,						
	PlotFunction = nil,					  
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_GOLD_SPENDING", 
	Concept2 = "CONCEPT_DIPLOMACY",
	Concept3 = "CONCEPT_CITY_PURCHASEITEM",

};

------------------------------------------------------------
------------------------------------------------------------

function CanCityBombardCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end
	
	if (not bEverCityBombard) then
		for v in player:Cities() do
			if (v:CanRangeStrikeNow()) then
				return ACTIVE;
			end
		end		
	end
	
	return INACTIVE;
end

function GetBombardCity ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return nil;	
	end
	
	for v in player:Cities() do
		if (v:CanRangeStrikeNow()) then
			return v:Plot();
		end
	end	
	
	return nil;
end

Tutorial { 
	ID = "CITY_BOMBARD",				
	ActivateButtonText = true,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = CanCityBombardCheck,			
	UnitIndexFunction = nil,						
	PlotFunction = GetBombardCity,		  
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_DEFENDING",
	Concept2 = "CONCEPT_CITY_COMBAT",
	Concept3 = "CONCEPT_COMBAT_CITY_BOMBARD",

};

------------------------------------------------------------
------------------------------------------------------------

function RightClickCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;	
	end

	if (Game.IsEverRightClickMoved()) then
		return COMPLETE;
	end

	if (Game.GetGameTurn() >= 20) then
		return ACTIVE;
	end
		
	return INACTIVE;
end

function GetRightClickCheckUnit ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return -1;	
	end

	for v in player:Units() do
		if (v ~= nil) then
			if (v:IsReadyToMove()) then
				return v:GetID();
			end
		end
	end

	return -1;	
end

Tutorial { 
	ID = "RIGHT_CLICK_MOVE",			
	ActivateButtonText = false,	
	Advisor = ADVISOR_SCIENCE,  
	CheckFunction = RightClickCheck,				
	UnitIndexFunction = GetRightClickCheckUnit,		
	PlotFunction = nil,					  
	MinTurn =  0, 
	MinTurnTime = 10, 
	TurnsCheck =  0, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	Concept1 = "CONCEPT_MOVEMENT_RIGHT_CLICK",
	Concept2 = "CONCEPT_MOVEMENT_HOW_TO_ORDER",
	Concept3 = "CONCEPT_MOVEMENT",

};

------------------------------------------------------------
------------------------------------------------------------
local pForestRevealedPlot   = nil;
local pHillRevealedPlot     = nil;
local pMarshRevealedPlot    = nil;
local pMountainRevealedPlot = nil;
local pJungleRevealedPlot	= nil;
local pRiverRevealedPlot	= nil;
function OnHexFogEvent( hexPos, fowType, bWholeMap )

	local BlackFog = 0; -- invisible
	local GreyFog  = 1; -- once seen
	local WhiteFog = 2; -- eyes on

	if (fowType == WhiteFog) then
		local gridPosX, gridPosY = ToGridFromHex( hexPos.x, hexPos.y );
		local pPlot = Map.GetPlot( gridPosX, gridPosY );	
		if (pPlot ~= nil) then
			if (pPlot:GetFeatureType() == FeatureTypes.FEATURE_FOREST) then
				pForestRevealedPlot = pPlot;
			elseif (pPlot:GetFeatureType() == FeatureTypes.FEATURE_MARSH) then
				pMarshRevealedPlot = pPlot;
			elseif (pPlot:GetFeatureType() == FeatureTypes.FEATURE_JUNGLE) then
				pJungleRevealedPlot = pPlot;
			end
				
			if (pPlot:IsHills()) then
				pHillRevealedPlot = pPlot;
			elseif (pPlot:IsMountain()) then
				pMountainRevealedPlot = pPlot;
			end
			
			if (pPlot:IsRiver()) then
				pRiverRevealedPlot = pPlot;
			end
		end
	end
end
Events.HexFOWStateChanged.Add( OnHexFogEvent );

function MovementForest ()
	if (pForestRevealedPlot ~= nil) then
		return ACTIVE;
	end

	return INACTIVE;
end

function FindMovementForest ()
	return pForestRevealedPlot;
end

Tutorial { 
	ID = "MOVEMENT_FOREST",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = MovementForest,   
	UnitIndexFunction = nil, 
	PlotFunction = FindMovementForest,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	MaxInQueue = 0,	
	Concept1 = "CONCEPT_COMBAT_BONUSES_TERRAIN",
	Concept2 = "CONCEPT_TERRAIN_FEATURE_FORESTS",
	Concept3 = "CONCEPT_WORKERS_CLEARINGLAND",

};

------------------------------------------------------------
------------------------------------------------------------
function MovementHill ()
	if (pHillRevealedPlot ~= nil) then
		return ACTIVE;
	end

	return INACTIVE;
end

function FindMovementHill ()
	return pHillRevealedPlot;
end

Tutorial { 
	ID = "MOVEMENT_HILLS",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = MovementHill,   
	UnitIndexFunction = nil, 
	PlotFunction = FindMovementHill,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	MaxInQueue = 0,	
	Concept1 = "CONCEPT_COMBAT_BONUSES_TERRAIN",
	Concept2 = "CONCEPT_TERRAIN_HILLS",
	Concept3 = "CONCEPT_FOG_OF_WAR_OBSCURING_TERRAIN",

};

------------------------------------------------------------
------------------------------------------------------------
function MovementMarsh ()
	if (pMarshRevealedPlot ~= nil) then
		return ACTIVE;
	end

	return INACTIVE;
end

function FindMovementMarsh ()
	return pMarshRevealedPlot;
end

Tutorial { 
	ID = "MOVEMENT_MARSH",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = MovementMarsh,   
	UnitIndexFunction = nil, 
	PlotFunction = FindMovementMarsh,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	MaxInQueue = 0,	
	Concept1 = "CONCEPT_COMBAT_BONUSES_TERRAIN",
	Concept2 = "CONCEPT_TERRAIN_FEATURE_MARSH",
	Concept3 = "CONCEPT_WORKERS_CLEARINGLAND",

};

------------------------------------------------------------
------------------------------------------------------------
function MovementMountain ()
	if (pMountainRevealedPlot ~= nil) then
		return ACTIVE;
	end

	return INACTIVE;
end

function FindMovementMountain ()
	return pMountainRevealedPlot;
end

Tutorial { 
	ID = "MOVEMENT_MOUNTAIN",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = MovementMountain,   
	UnitIndexFunction = nil, 
	PlotFunction = FindMovementMountain,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	MaxInQueue = 0,	
	Concept1 = "CONCEPT_COMBAT_BONUSES_TERRAIN",
	Concept2 = "CONCEPT_TERRAIN_MOUNTAINS",
	Concept3 = "CONCEPT_FOG_OF_WAR_OBSCURING_TERRAIN",

};

------------------------------------------------------------
------------------------------------------------------------
function MovementJungle ()
	if (pJungleRevealedPlot ~= nil) then
		return ACTIVE;
	end

	return INACTIVE;
end

function FindMovementJungle ()
	return pJungleRevealedPlot;
end

Tutorial { 
	ID = "MOVEMENT_JUNGLE",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = MovementJungle,   
	UnitIndexFunction = nil, 
	PlotFunction = FindMovementJungle,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,
	MaxInQueue = 0,		
	Concept1 = "CONCEPT_COMBAT_BONUSES_TERRAIN",
	Concept2 = "CONCEPT_TERRAIN_FEATURE_JUNGLE",
	Concept3 = "CONCEPT_WORKERS_CLEARINGLAND",

};

------------------------------------------------------------
------------------------------------------------------------
function MovementRiver ()
	if (pRiverRevealedPlot ~= nil) then
		return ACTIVE;
	end

	return INACTIVE;
end

function FindMovementRiver ()
	return pRiverRevealedPlot;
end

Tutorial { 
	ID = "MOVEMENT_RIVER",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = MovementRiver,   
	UnitIndexFunction = nil, 
	PlotFunction = FindMovementRiver,				      
	MinTurn =  1, 
	TurnsCheck = 1, 
	TutorialLevel = 0,	
	MaxInQueue = 0,	
	Concept1 = "CONCEPT_COMBAT_BONUSES_TERRAIN",
	Concept2 = "CONCEPT_TERRAIN_RIVERS",
	Concept3 = "CONCEPT_GOLD_TERRAIN",

};

------------------------------------------------------------
------------------------------------------------------------

function FirstCombatNavalUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:GetDomainType() == DomainTypes.DOMAIN_SEA and v:IsCanAttack()) then
			return ACTIVE;
		end
	end

	return INACTIVE;
end

function GetCombatNavalUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	for v in player:Units() do
		if (v:GetDomainType() == DomainTypes.DOMAIN_SEA and v:IsCanAttack()) then
			return v:GetID();
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "COMBAT_NAVAL_UNIT",				
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = FirstCombatNavalUnit,				
	UnitIndexFunction = GetCombatNavalUnit,				
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_COMBAT_NAVAL",
	Concept2 = "CONCEPT_COMBAT_RANGED_COMBAT",
	Concept3 = "CONCEPT_MOVEMENT_NAVAL",

};

------------------------------------------------------------
------------------------------------------------------------

function FirstWorkerNavalUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:GetDomainType() == DomainTypes.DOMAIN_SEA and v:WorkRate() > 0) then
			return ACTIVE;
		end
	end

	return INACTIVE;
end

function GetWorkerNavalUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	for v in player:Units() do
		if (v:GetDomainType() == DomainTypes.DOMAIN_SEA and v:WorkRate() > 0) then
			return v:GetID();
		end
	end
	
	return -1;
end

Tutorial { 
	ID = "WORKER_NAVAL_UNIT",				
	ActivateButtonText = true,  
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = FirstWorkerNavalUnit,				
	UnitIndexFunction = GetWorkerNavalUnit,				
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_COMBAT_WHICH_UNITS_CAN_FIGHT",
	Concept2 = "CONCEPT_WORKERS_WORKBOATS",
	Concept3 = "CONCEPT_MOVEMENT_NAVAL",

};

------------------------------------------------------------
------------------------------------------------------------

function FirstSiegeUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return INACTIVE;	
	end

	for v in player:Units() do
		if (v:GetDomainType() == DomainTypes.DOMAIN_LAND and v:IsCanAttack() and v:IsMustSetUpToRangedAttack()) then
			return ACTIVE;
		end
	end

	return INACTIVE;
end

function GetSiegeUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	for v in player:Units() do
		if (v:GetDomainType() == DomainTypes.DOMAIN_LAND and v:IsCanAttack() and v:IsMustSetUpToRangedAttack()) then
			return v:GetID();
		end
	end
	
	return -1;
end

Tutorial { 
	ID =  "SIEGE_UNIT",				
	ActivateButtonText = true,  
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = FirstSiegeUnit,				
	UnitIndexFunction = GetSiegeUnit,				
	PlotFunction = nil,				      
	MinTurn =  1, 
	MinTurnTime = -1, 
	TurnsCheck = 0, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_ATTACKING_RANGED",
	Concept2 = "CONCEPT_CITY_CONQUERING",
	Concept3 = "CONCEPT_COMBAT_SIEGE",

};

------------------------------------------------------------
------------------------------------------------------------
local pNaturalWonderPlot = nil;
function DiscoveredNaturalWonder (i, j)
	local iX, iY = ToGridFromHex(i, j);
	pNaturalWonderPlot = Map.GetPlot(iX, iY);
end
Events.NaturalWonderRevealed.Add( DiscoveredNaturalWonder );


function DiscoveredNaturalWonderCheck ()
	if (pNaturalWonderPlot) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

function DiscoveredNaturalWonderPlot ()
	return pNaturalWonderPlot;
end

Tutorial { 
	ID = "DISCOVERED_NATURAL_WONDER",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN, 
	CheckFunction = DiscoveredNaturalWonderCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = DiscoveredNaturalWonderPlot,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_TERRAIN_FEATURE_NATURAL_WONDER",
	Concept2 = "CONCEPT_CULTURE_GAINING",
	Concept3 = "CONCEPT_HAPPINESS",
};

------------------------------------------------------------
------------------------------------------------------------

function GetTrespassingUnit ()
	local pPlayer = GetPlayer();
	local pTeam = Teams[pPlayer:GetTeam()];
	for v in pPlayer:Units() do
		if (v:IsCombatUnit()) then
			local pPlot = v:GetPlot();
			local iPlotOwner = pPlot:GetOwner();
			local iPlotTeam = -1;
			if (iPlotOwner >= 0) then
				iPlotTeam = Players[iPlotOwner]:GetTeam();
			end
			
			if (iPlotOwner >= 0 and Players[iPlotOwner]:IsMinorCiv() and not pTeam:IsAtWar(iPlotTeam)) then
				if (Players[iPlotOwner]:GetMinorCivFriendshipLevelWithMajor(pPlayer:GetID()) < 1) then
					return v;
				end			
			end
		end
	end
	
	return nil;
end

function CityStatesTrespassingCheck ()
	if (GetTrespassingUnit() ~= nil) then
		return ACTIVE;
	end
		
	return INACTIVE;
end

function CityStatesTrespassingUnit ()
	local unit = GetTrespassingUnit();
	
	if (unit) then
		return unit:GetID();
	end
	
	return -1;
end

Tutorial { 
	ID = "CITY_STATES_TRESPASSING",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN, 
	CheckFunction = CityStatesTrespassingCheck,   
	UnitIndexFunction = CityStatesTrespassingUnit, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_STATE",
	Concept2 = "CONCEPT_CITY_STATE_INFLUENCE",
	Concept3 = "",
};

------------------------------------------------------------
------------------------------------------------------------
function GetEngageDiplomacyPlayer ()
	local pPlayer = GetPlayer();
	local pTeam = Teams[pPlayer:GetTeam()];
	
	-- Loop through all the players the active player knows
	for iPlayerLoop = 0, GameDefines.MAX_CIV_PLAYERS-1, 1 do
		local pOtherPlayer = Players[iPlayerLoop];
		local iOtherTeam = pOtherPlayer:GetTeam();
		-- Valid player? - Can't be us, and has to be alive
		if (pPlayer:GetTeam() ~= iOtherTeam and pOtherPlayer:IsAlive()) then			
			-- Met this player?
			if (pTeam:IsHasMet(iOtherTeam)) then
				for v in pOtherPlayer:Cities() do
					local pPlot = v:Plot();
					if (pPlot:IsRevealed(pPlayer:GetTeam())) then
						return pOtherPlayer;
					end
				end	
			end
		end
	end
end

function HowToEngageDiplomacyCheck ()
	if (GetEngageDiplomacyPlayer() ~= nil) then
		return ACTIVE;
	end
		
	return INACTIVE;
end

function HowToEngageDiplomacyPlot ()
	local pPlayer = GetPlayer();
	
	local pOtherPlayer = GetEngageDiplomacyPlayer();
	if (pOtherPlayer) then
		for v in pOtherPlayer:Cities() do
			local pPlot = v:Plot();
			if (pPlot:IsRevealed(pPlayer:GetTeam())) then
				return pPlot;
			end
		end	
	end
		
	return nil;
end

Tutorial { 
	ID = "HOW_TO_ENGAGE_DIPLOMACY",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_FOREIGN, 
	CheckFunction = HowToEngageDiplomacyCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = HowToEngageDiplomacyPlot,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITY_STATE_COMMUNICATION",
	Concept2 = "CONCEPT_VICTORY_VICTORY_DIPLOMATIC",
	Concept3 = "CONCEPT_DIPLOMACY",
};

------------------------------------------------------------
------------------------------------------------------------
function EndTurnCheck ()
	if (Game.GetGameTurn() > 0) then
		return COMPLETE;
	end

	local pPlayer = GetPlayer();
	if (pPlayer:GetEndTurnBlockingType() == EndTurnBlockingTypes.NO_ENDTURN_BLOCKING_TYPE) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "END_TURN";
	ActivateButtonText = false;
	Advisor = ADVISOR_SCIENCE,
	CheckFunction = EndTurnCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = 1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 0;
	Concept1 = "";
	Concept2 = "";
	Concept3 = "";
}

------------------------------------------------------------
------------------------------------------------------------
function HowToWinCheck ()
	if (Game.GetStartEra() == 0) then
		local pPlayer = GetPlayer();
		if (pPlayer:GetCurrentEra() == 3) then
			return ACTIVE;
		end
	else
		if (Game.GetGameTurn() - Game.GetStartTurn() > 10) then
			return ACTIVE;	
		end
	end
		
	return INACTIVE;
end

Tutorial {
	ID = "WINNING_GAME";
	ActivateButtonText = false;
	Advisor = ADVISOR_SCIENCE,
	CheckFunction = HowToWinCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_VICTORY";
	Concept2 = "CONCEPT_VICTORY_DEFEAT";
	Concept3 = "CONCEPT_VICTORY_END_OF_TIME";
}

------------------------------------------------------------
------------------------------------------------------------
function GetGreatGeneralUnit ()
	local player = GetPlayer();
	if (player == nil) then
		return -1;	
	end

	if(ggUnitClassID ~= nil) then
		for v in player:Units() do
			if(v:GetUnitClassType() == ggUnitClassID) then
				return v:GetID();
			end
		end
	end
	
	return -1;
end

function GreatGeneralCheck ()
	if (GetGreatGeneralUnit() ~= -1) then
		return ACTIVE;
	end
	
	return INACTIVE;	
end

Tutorial {
	ID = "GREAT_GENERAL";
	ActivateButtonText = true;
	Advisor = ADVISOR_MILITARY,
	CheckFunction = GreatGeneralCheck;
	UnitIndexFunction = GetGreatGeneralUnit;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_SPECIALISTS_AND_GREAT_PEOPLE_GREAT_PEOPLE_GREAT_GENERAL";
	Concept2 = "CONCEPT_SPECIALISTS_AND_GREAT_PEOPLE_GREAT_PEOPLE";
	Concept3 = "CONCEPT_SPECIALISTS_AND_GREAT_PEOPLE_GREAT_PEOPLE_COMBAT_BONUS";
}

------------------------------------------------------------
------------------------------------------------------------

local CAMP_TYPE = GameInfoTypes["IMPROVEMENT_BARBARIAN_CAMP"];
local pBarbarianCampPlot = nil;

function OnImprovementCreated (HPosX, HPosY, CultureType, ContinentType, PlayerType, engineImprovementTypeDoNotUse, ImprovementType, engineResourceTypeDoNotUse, RawResourceType, ImprovementEra, ImprovementState)
	--print("OnImprovementCreated");
	--print("HPosX: " .. HPosX);
	--print("HPosY: " .. HPosY);
	--print("CultureType: " .. CultureType);
	--print("PlayerType: " .. PlayerType);
	--print("ImprovementType: " .. ImprovementType);
	--print("RawResourceType: " .. RawResourceType);
	--print("ImprovementEra: " .. ImprovementEra);
	--print("ImprovementState: " .. ImprovementState);
	--print("------------------");
		
	local iX, iY = ToGridFromHex( HPosX, HPosY );
	local pPlot = Map.GetPlot(iX, iY);

	--print("X: " .. iX);
	--print("Y: " .. iY);
	
	if (ImprovementType == CAMP_TYPE) then
		pBarbarianCampPlot = pPlot;
	end	
end
Events.SerialEventImprovementCreated.Add( OnImprovementCreated );

function GetBarbarianCampPlot ()
	return pBarbarianCampPlot;
end

function BarbarianCampCheck ()
	if (pBarbarianCampPlot and pBarbarianCampPlot:HasBarbarianCamp()) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "BARBARIAN_CAMP";
	ActivateButtonText = true;
	Advisor = ADVISOR_MILITARY,
	CheckFunction = BarbarianCampCheck;
	UnitIndexFunction = nil;
	PlotFunction = GetBarbarianCampPlot;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_BARBARIANS_ENCAMPMENTS";
	Concept2 = "CONCEPT_BARBARIANS";
	Concept3 = "CONCEPT_GOLD_BARBARIAN_CAMPS";
}

------------------------------------------------------------
------------------------------------------------------------
function HappinessCheck ()
	local pPlayer = GetPlayer();

	for v in pPlayer:Cities() do
		if (v:GetPopulation() >= 3) then
			return ACTIVE;
		end
	end	
	
	return INACTIVE;
end

Tutorial {
	ID = "HAPPINESS";
	ActivateButtonText = false;
	Advisor = ADVISOR_ECONOMIC,
	CheckFunction = HappinessCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_HAPPINESS";
	Concept2 = "CONCEPT_FOOD_AND_CITY_GROWTH_UNHAPPINESS";
	Concept3 = "CONCEPT_RESOURCES_LUXURY";
}

------------------------------------------------------------
------------------------------------------------------------
function ExcessLuxuryResourceCheck ()
	local pPlayer = GetPlayer();
	
	local luxoryResources = {};
	
	local i = 1;
	for resource in GameInfo.Resources("ResourceClassType = 'RESOURCECLASS_LUXURY'") do
		luxoryResources[i] = resource.ID;
		i = i + 1;
	end
	
	local whatWeHave = {};
	local whatWeCanTrade = {};
	for i,v in ipairs(luxoryResources) do
		local amount = pPlayer:GetNumResourceAvailable(iResourceLoop, false);
		whatWeHave[v] = amount;
		if(amount > 1) then
			table.insert(whatWeCanTrade, v);
		end
	end
	
	if(#whatWeCanTrade > 0) then
		local team = pPlayer:GetTeam();
		local pTeam = Teams[team];

		for iPlayerLoop = 0, GameDefines.MAX_MAJOR_CIVS, 1 do

			local pOtherPlayer = Players[iPlayerLoop];
			local iOtherTeam = pOtherPlayer:GetTeam();

			-- Valid player? - Can't be us, and has to be alive
			if (team ~= iOtherTeam and pOtherPlayer:IsAlive()) then			

				-- Met this player?
				if (pTeam:IsHasMet(iOtherTeam) and not pTeam:IsAtWar(iOtherTeam)) then
					
					for i, tradeableResource in ipairs(whatWeCanTrade) do
						-- if they have none of the resource
						if (pOtherPlayer:GetNumResourceAvailable(tradeableResource, true) == 0) then
							--print("They don't have our resource");
							-- find a resource we need
							for li, luxoryResource in ipairs(luxoryResources) do
								if(whatWeHave[luxoryResource] == 0) then
									if(pOtherPlayer:GetNumResourceAvailable(tradeableResource, false) > 0) then
										return ACTIVE;
									end
								end 		
							end
						end
					end		
				end
			end
		end
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "EXCESS_LUXURY_RESOURCE";
	ActivateButtonText = false;
	Advisor = ADVISOR_FOREIGN,
	CheckFunction = ExcessLuxuryResourceCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = 1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_RESOURCES_LUXURY";
	Concept2 = "CONCEPT_DIPLOMACY_TRADE";
	Concept3 = "CONCEPT_DIPLOMACY_RESOURCES";
}

------------------------------------------------------------
------------------------------------------------------------

function BankruptCheck ()
	local pPlayer = GetPlayer();
	if (pPlayer:GetGold() < 0) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "BANKRUPTCY";
	ActivateButtonText = false;
	Advisor = ADVISOR_ECONOMIC,
	CheckFunction = BankruptCheck;
	UnitIndexFunction = nil;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_GOLD_RUNNING_DEFICIT";
	Concept2 = "CONCEPT_GOLD_WHERE_TO_GET";
	Concept3 = "";
}

------------------------------------------------------------
------------------------------------------------------------
function GetOccupiedCity (bCheckProduction)
	local pPlayer = GetPlayer();
	
	for v in pPlayer:Cities() do
		if (v:IsOccupied() and not v:IsNoOccupiedUnhappiness()) then
			if (bCheckProduction) then
				if (v:GetProduction() == 0) then
					return v;
				end
			else
				return v;			
			end
		end
	end
	
	return nil;
end

function RecommendCourthouseCheck ()
	local pCity = GetOccupiedCity(true);
	if (pCity) then
		for building in GameInfo.Buildings() do
		 	local buildingID = building.ID;
			if (building.NoOccupiedUnhappiness and not pCity:IsHasBuilding(buildingID) and pCity:GetProductionBuilding() ~= buildingID) then
				--print("Found building that addresses occupied happiness");
 				if (pCity:CanConstruct(buildingID)) then
 					--print("Can build building. Active!");
					return ACTIVE;	 			
 				end				
			end
		end
	end
	
	return INACTIVE;	
end

function GetBuildCourthouseCityPlot ()
	local pCity = GetOccupiedCity(true);
	if (pCity) then
		return pCity:Plot();
	end
	
	return nil;
end

Tutorial {
	ID = "RECOMMEND_COURTHOUSE";
	ActivateButtonText = true;
	Advisor = ADVISOR_ECONOMIC,
	CheckFunction = RecommendCourthouseCheck;
	UnitIndexFunction = nil;
	PlotFunction = GetBuildCourthouseCityPlot;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_HAPPINESS";
	Concept2 = "CONCEPT_CITY_ANNEX";
	Concept3 = "";
}

function ResearchCourthouseCheck ()
	local pPlayer = GetPlayer();
	if (pPlayer:GetCurrentResearch() ~= -1) then
		return INACTIVE;
	end

	local pCity = GetOccupiedCity(false);
	if (pCity) then
		for building in GameInfo.Buildings() do
			if (building.NoOccupiedUnhappiness) then
 				local buildingID = building.ID;
 				if (not pCity:CanConstruct(buildingID) and not pCity:IsHasBuilding(buildingID) and pCity:GetProductionBuilding() ~= buildingID) then
					return ACTIVE;	 			
 				end				
			end
		end
	end
	
	return INACTIVE;	
end

function GetResearchCourthouseCityPlot ()
	local pCity = GetOccupiedCity(false);
	if (pCity) then
		return pCity:Plot();
	end
	
	return nil;
end

Tutorial {
	ID = "RESEARCH_COURTHOUSE";
	ActivateButtonText = true;
	Advisor = ADVISOR_ECONOMIC;
	CheckFunction = ResearchCourthouseCheck;
	UnitIndexFunction = nil;
	PlotFunction = GetResearchCourthouseCityPlot;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_HAPPINESS";
	Concept2 = "CONCEPT_CITY_ANNEX";
	Concept3 = "";
}

------------------------------------------------------------
------------------------------------------------------------
function GetCSFriendPlot ()
	local pPlayer = GetPlayer();
	local pTeam = Teams[pPlayer:GetTeam()];
	
	for iPlayerLoop = GameDefines.MAX_MAJOR_CIVS, GameDefines.MAX_CIV_PLAYERS - 1, 1 do	
		local pOtherPlayer = Players[iPlayerLoop];
		local iOtherTeam = pOtherPlayer:GetTeam();
		if (pPlayer:GetTeam() ~= iOtherTeam and pOtherPlayer:IsAlive() and pOtherPlayer:IsFriends()) then
			for v in pOtherPlayer:Cities() do
				local pPlot = v:Plot();
				if (pPlot:IsRevealed(pPlayer:GetTeam())) then
					return pPlot;
				end
			end
		end
	end
	
	return nil;
end


function MadeCSFriendCheck ()
	local pPlot = GetCSFriendPlot();
	if (pPlot) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "CITY_STATE_FRIEND";
	ActivateButtonText = true;
	Advisor = ADVISOR_FOREIGN;
	CheckFunction = MadeCSFriendCheck;
	UnitIndexFunction = nil;
	PlotFunction = GetCSFriendPlot;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_CITY_STATE_INFLUENCE_LEVELS_FRIENDS";
	Concept2 = "CONCEPT_CITY_STATE_INFLUENCE";
	Concept3 = "CONCEPT_CITY_STATE_COMMUNICATION";
}

------------------------------------------------------------
------------------------------------------------------------
function ZoneOfControlPlot ()
	local player = GetPlayer();
	for v in player:Units() do
		if (v:IsSelected() and not v:IsBusy()) then
			if (v:GetNumEnemyUnitsAdjacent(v) > 0 or v:IsEnemyCityAdjacent()) then
				return v:GetPlot();
			end
		end
	end

	return nil;
	
end

function ZoneOfControlCheck ()
	if (ZoneOfControlPlot()) then
		return ACTIVE;
	end
	
	return INACTIVE;
end



Tutorial {
	ID = "ZONE_OF_CONTROL";
	ActivateButtonText = true;
	Advisor = ADVISOR_SCIENCE;
	CheckFunction = ZoneOfControlCheck;
	UnitIndexFunction = nil;
	PlotFunction = ZoneOfControlPlot;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 3;
	Concept1 = "CONCEPT_MOVEMENT_COMBAT_ZONES_OF_CONTROL";
	Concept2 = "CONCEPT_MOVEMENT_COMBAT";
	Concept3 = "";		
}

------------------------------------------------------------
------------------------------------------------------------
function GreatPersonUnit ()
	local player = GetPlayer();
	
	for v in player:Units() do
		if (v:IsSelected() and not v:IsBusy() and v:GetSpecialUnitType() ~= -1 and not v:IsCombatUnit() and v:GetUnitClassType() ~= ggUnitClassID and v:GetDomainType() == DomainTypes.DOMAIN_LAND) then
			return v:GetID();
		end
	end
	
	return -1;
end

function GreatPersonCheck ()
	if (GreatPersonUnit() ~= -1) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

Tutorial {
	ID = "GREAT_PERSON";
	ActivateButtonText = true;
	Advisor = ADVISOR_FOREIGN;
	CheckFunction = GreatPersonCheck;
	UnitIndexFunction = GreatPersonUnit;
	PlotFunction = nil;
	MinTurn = 0;
	MinTurnTime = -1;
	TurnsCheck = -1;
	PesterAgainTurn = -1;
	TutorialLevel = 1;
	Concept1 = "CONCEPT_SPECIALISTS_AND_GREAT_PEOPLE_GREAT_PEOPLE";
	Concept2 = "CONCEPT_SPECIALISTS_AND_GREAT_PEOPLE_GREAT_PEOPLE_GENERATING";
	Concept3 = "CONCEPT_SPECIALISTS_AND_GREAT_PEOPLE_GREAT_PEOPLE_ABILITIES";		
}

------------------------------------------------------------
------------------------------------------------------------
local CombatStartedInfo = {};
function CombatStarted (iAttackingPlayer, iAttackingUnit, iAttackingUnitDamage, iAttackingUnitFinalDamage, iAttackingUnitMaxHitPoints, iDefendingPlayer, iDefendingUnit, iDefendingUnitDamage, iDefendingUnitFinalDamage, iDefendingUnitMaxHitPoints)
	--print ("CombatStarted");
	CombatStartedInfo = {};
	CombatStartedInfo.iAttackingPlayer = iAttackingPlayer;
	CombatStartedInfo.iAttackingUnit = iAttackingUnit;
	CombatStartedInfo.iAttackingUnitDamage = iAttackingUnitDamage;
	CombatStartedInfo.iAttackingUnitFinalDamage = iAttackingUnitFinalDamage;
	CombatStartedInfo.iAttackingUnitMaxHitPoints = iAttackingUnitMaxHitPoints;
	
	CombatStartedInfo.iDefendingPlayer = iDefendingPlayer;
	CombatStartedInfo.iDefendingUnit   = iDefendingUnit;
	CombatStartedInfo.iDefendingUnitDamage = iDefendingUnitDamage;
	CombatStartedInfo.iDefendingUnitFinalDamage = iDefendingUnitFinalDamage;
	CombatStartedInfo.iDefendingUnitMaxHitPoints = iDefendingUnitMaxHitPoints;
	
	--print ("iAttackingPlayer:           " .. CombatStartedInfo.iAttackingPlayer);
	--print ("iAttackingUnit:             " .. CombatStartedInfo.iAttackingUnit);
	--print ("iAttackingUnitDamage:       " .. CombatStartedInfo.iAttackingUnitDamage);
	--print ("iAttackingUnitFinalDamage:  " .. CombatStartedInfo.iAttackingUnitFinalDamage);
	--print ("iAttackingUnitMaxHitPoints: " .. CombatStartedInfo.iAttackingUnitMaxHitPoints);
	--print ("iDefendingPlayer:		    " .. CombatStartedInfo.iDefendingPlayer);
	--print ("iDefendingUnit:			    " .. CombatStartedInfo.iDefendingUnit);
	--print ("iDefendingUnitDamage:       " .. CombatStartedInfo.iDefendingUnitDamage);
	--print ("iDefendingUnitFinalDamage:  " .. CombatStartedInfo.iDefendingUnitFinalDamage);	
	--print ("iDefendingUnitMaxHitPoints: " .. CombatStartedInfo.iDefendingUnitMaxHitPoints);
end
Events.RunCombatSim.Add( CombatStarted );

local CombatEndedInfo = {};
function CombatEnded (iAttackingPlayer, iAttackingUnit, iAttackingUnitDamage, iAttackingUnitFinalDamage, iAttackingUnitMaxHitPoints, iDefendingPlayer, iDefendingUnit, iDefendingUnitDamage, iDefendingUnitFinalDamage, iDefendingUnitMaxHitPoints)
	--print ("CombatEnded");
	CombatEndedInfo = {};
	CombatEndedInfo.iAttackingPlayer = iAttackingPlayer;
	CombatEndedInfo.iAttackingUnit = iAttackingUnit;
	CombatEndedInfo.iAttackingUnitDamage = iAttackingUnitDamage;
	CombatEndedInfo.iAttackingUnitFinalDamage = iAttackingUnitFinalDamage;
	CombatEndedInfo.iAttackingUnitMaxHitPoints = iAttackingUnitMaxHitPoints;
	
	CombatEndedInfo.iDefendingPlayer = iDefendingPlayer;
	CombatEndedInfo.iDefendingUnit   = iDefendingUnit;
	CombatEndedInfo.iDefendingUnitDamage = iDefendingUnitDamage;
	CombatEndedInfo.iDefendingUnitFinalDamage = iDefendingUnitFinalDamage;
	CombatEndedInfo.iDefendingUnitMaxHitPoints = iDefendingUnitMaxHitPoints;
	
	--print ("iAttackingPlayer:           " .. CombatEndedInfo.iAttackingPlayer);
	--print ("iAttackingUnit:             " .. CombatEndedInfo.iAttackingUnit);
	--print ("iAttackingUnitDamage:       " .. CombatEndedInfo.iAttackingUnitDamage);
	--print ("iAttackingUnitFinalDamage:  " .. CombatEndedInfo.iAttackingUnitFinalDamage);
	--print ("iAttackingUnitMaxHitPoints: " .. CombatEndedInfo.iAttackingUnitMaxHitPoints);
	--print ("iDefendingPlayer:		    " .. CombatEndedInfo.iDefendingPlayer);
	--print ("iDefendingUnit:			    " .. CombatEndedInfo.iDefendingUnit);
	--print ("iDefendingUnitDamage:       " .. CombatEndedInfo.iDefendingUnitDamage);
	--print ("iDefendingUnitFinalDamage:  " .. CombatEndedInfo.iDefendingUnitFinalDamage);	
	--print ("iDefendingUnitMaxHitPoints: " .. CombatEndedInfo.iDefendingUnitMaxHitPoints);

end
Events.EndCombatSim.Add( CombatEnded );

------------------------------------------------------------
------------------------------------------------------------

function AttackerSurvived (CombatInfo)
	if (CombatInfo.iAttackingUnitFinalDamage < CombatInfo.iAttackingUnitMaxHitPoints) then
		return true;
	end
	
	return false;
end

function AttackerIsUnit (CombatInfo)
	if (CombatInfo.iAttackingUnit ~= -1) then
		return true;
	end
	
	return false;
end

function DefenderIsUnit (CombatInfo)
	if (CombatInfo.iDefendingUnit ~= -1) then
		return true;
	end
	
	return false;
end

function DefenderSurvived (CombatInfo)
	if (CombatInfo.iDefendingUnitFinalDamage < CombatInfo.iDefendingUnitMaxHitPoints) then
		return true;
	end
	
	return false;
end

function GoodAttackCheck ()
	if (CombatEndedInfo.iAttackingPlayer == Game.GetActivePlayer() and AttackerIsUnit(CombatEndedInfo)) then
		if (AttackerSurvived(CombatEndedInfo) and DefenderIsUnit(CombatEndedInfo)) then
			if (CombatEndedInfo.iAttackingUnitDamage > CombatEndedInfo.iDefendingUnitDamage) then
				return ACTIVE;
			end
		end
	end
	
	return INACTIVE;
end

function BadAttackCheck ()
	if (CombatEndedInfo.iAttackingPlayer == Game.GetActivePlayer() and AttackerIsUnit(CombatEndedInfo)) then
		if (DefenderSurvived(CombatEndedInfo) and DefenderIsUnit(CombatEndedInfo)) then
			if (CombatEndedInfo.iAttackingUnitDamage < CombatEndedInfo.iDefendingUnitDamage) then
				return ACTIVE;
			end
		end
	end
	
	return INACTIVE;
end

function NeutralAttackCheck ()
	if (CombatEndedInfo.iAttackingPlayer == Game.GetActivePlayer() and AttackerIsUnit(CombatEndedInfo)) then
		if (AttackerSurvived(CombatEndedInfo) and DefenderSurvived(CombatEndedInfo) and DefenderIsUnit(CombatEndedInfo)) then
			if (CombatEndedInfo.iAttackingUnitDamage == CombatEndedInfo.iDefendingUnitDamage) then
				return ACTIVE;
			end
		end
	end
	
	return INACTIVE;
end

Tutorial { 
	ID = "GOOD_ATTACK_2",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = GoodAttackCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_COMBAT",
	Concept2 = "CONCEPT_COMBAT_BONUSES",
	Concept3 = "CONCEPT_COMBAT_EXPERIENCE_ACCUMULATING_COMBAT",
};

Tutorial { 
	ID = "BAD_ATTACK_2",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = BadAttackCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_COMBAT",
	Concept2 = "CONCEPT_COMBAT_BONUSES",
	Concept3 = "CONCEPT_COMBAT_EXPERIENCE_ACCUMULATING_COMBAT",
};

Tutorial {
	ID = "NEUTRAL_ATTACK_2",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = NeutralAttackCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_COMBAT",
	Concept2 = "CONCEPT_COMBAT_BONUSES",
	Concept3 = "CONCEPT_COMBAT_EXPERIENCE_ACCUMULATING_COMBAT",
};

------------------------------------------------------------
------------------------------------------------------------
function TutorialBuildCityCheatCheck ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (player:GetNumCities() > 0) then
		return ACTIVE;
	end
	
	return INACTIVE;
end

function TutorialBuildCityCheatPlot ()
	local player = GetPlayer();
	for v in player:Cities() do
		return v:Plot();
	end
end

Tutorial { 
	ID = "TUTORIAL_BUILD_CITY_CHEAT",		
	ActivateButtonText = true,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = TutorialBuildCityCheatCheck,   
	UnitIndexFunction = nil, 
	PlotFunction = TutorialBuildCityCheatPlot,				      
	MinTurn =  0, 
	MinTurnTime = -1, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 1,	
	Concept1 = "CONCEPT_CITIES",
	Concept2 = "",
	Concept3 = "",
};
------------------------------------------------------------
------------------------------------------------------------
function Tutorial1MovementIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial { 
	ID = "TUTORIAL_1_INTRO_MOVEMENT_AND_EXPLORATION",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_SCIENCE, 
	CheckFunction = Tutorial1MovementIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,				      
	MinTurn =  0, 
	MinTurnTime = 0, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 999,	
	Concept1 = "CONCEPT_MOVEMENT",
	Concept2 = "CONCEPT_FOG_OF_WAR",
	Concept3 = "CONCEPT_TERRAIN",

};

------------------------------------------------------------
------------------------------------------------------------
function Tutorial2FoundCitiesIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial {
	ID = "TUTORIAL_2_INTRO_FOUND_CITIES",
	ActivateButtonText = false,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = Tutorial2FoundCitiesIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,
	MinTurn =  0, 
	MinTurnTime = 5, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 999,	
	Concept1 = "CONCEPT_CITIES",
	Concept2 = "CONCEPT_FOUNDING_CITIES",
	Concept3 = "CONCEPT_WHERE_TO_FOUND_CITIES",

};

------------------------------------------------------------
------------------------------------------------------------
function Tutorial3ImprovingCitiesIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial { 
	ID = "TUTORIAL_3_INTRO_IMPROVING_CITIES_WITH_WORKERS",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_ECONOMIC, 
	CheckFunction = Tutorial3ImprovingCitiesIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,
	MinTurn =  0, 
	MinTurnTime = 0, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	Concept1 = "CONCEPT_CITY_WORKINGLAND",
	Concept2 = "CONCEPT_CITY_IMPROVINGLAND",
	Concept3 = "CONCEPT_WORKERS",

};

------------------------------------------------------------
------------------------------------------------------------
function Tutorial4CombatAndConquestIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial { 
	ID = "TUTORIAL_4_INTRO_COMBAT_AND_CONQUERING",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_MILITARY, 
	CheckFunction = Tutorial4CombatAndConquestIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,
	MinTurn =  0, 
	MinTurnTime = 0, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	Concept1 = "CONCEPT_COMBAT",
	Concept2 = "CONCEPT_BARBARIANS_ENCAMPMENTS",
	Concept3 = "CONCEPT_CITY_CONQUERING",

};

------------------------------------------------------------
------------------------------------------------------------
function Tutorial5DiplomacyIntro ()
	local player = GetPlayer();
	if (player == nil) then
		dprint("Error - player index not correct");		
		return INACTIVE;
	end

	if (Game.GetGameTurn() > 0) then
		return INVALID;
	end
	
	return ACTIVE;
end

Tutorial { 
	ID = "TUTORIAL_5_INTRO_DIPLOMACY",		
	ActivateButtonText = false,	
	Advisor = ADVISOR_FOREIGN, 
	CheckFunction = Tutorial5DiplomacyIntro,   
	UnitIndexFunction = nil, 
	PlotFunction = nil,
	MinTurn =  0, 
	MinTurnTime = 0, 
	TurnsCheck = -1, 
	PesterAgainTurn = -1,
	TutorialLevel = 0,	
	Concept1 = "CONCEPT_RESOURCES",
	Concept2 = "CONCEPT_DIPLOMACY",
	Concept3 = "CONCEPT_CITY_STATE",
};


------------------------------------------------------------
------------------------------------------------------------

function TutorialChecksEndTurn ()

	local player = GetPlayer();
	if (player == nil) then
		return;	
	end

	if (not EverHealedUnit) then	
		for v in player:Units() do
			if (v:GetActivityType() == ActivityTypes.ACTIVITY_HEAL) then
				EverHealedUnit = true;
			end
		end
	end
	
	if (not bEverCityBombard) then
		for v in player:Cities() do
			if (v:HasPerformedRangedStrikeThisTurn()) then
				bEverCityBombard = true;
			end
		end
	end
	
	if (iMaxThingsCityBuilt == 0) then
		for v in player:Cities() do
			if (v:GetNumThingsProduced() > iMaxThingsCityBuilt) then
				print("City building complete");
				iMaxThingsCityBuilt = v:GetNumThingsProduced();
			end
		end	
	end
	
	iLastTurnNumMinorsMet = GetPlayer():GetNumMinorCivsMet();
end

Tutorial {
	ID = "CITY_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_CITY_SCREEN_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_CITY_SCREEN_BODY",
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "DIPLO_TRADE_SCREEN",
	Advisor = ADVISOR_FOREIGN,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_LEADER_TRADE_DIALOG_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_LEADER_TRADE_DIALOG_BODY",
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CHOOSE_TECH_SCREEN",
	Advisor = ADVISOR_SCIENCE,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_TECH_CHOOSER_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_TECH_CHOOSER_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CHOOSETECH,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CHOOSE_PRODUCTION_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_PRODUCTION_CHOOSER_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_PRODUCTION_CHOOSER_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CHOOSEPRODUCTION,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CHOOSE_POLICY_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_SOCIAL_POLICY_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_SOCIAL_POLICY_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CHOOSEPOLICY,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "ECONOMIC_OVERVIEW_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_ECONOMIC_OVERVIEW_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_ECONOMIC_OVERVIEW_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_ECONOMIC_OVERVIEW,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "MILITARY_OVERVIEW_SCREEN",
	Advisor = ADVISOR_MILITARY,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_MILITARY_OVERVIEW_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_MILITARY_OVERVIEW_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_MILITARY_OVERVIEW,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "DIPLOMATIC_OVERVIEW_SCREEN",
	Advisor = ADVISOR_ECONOMIC,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_DIPLOMACY_OVERVIEW_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_DIPLOMACY_OVERVIEW_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_DIPLOMATIC_OVERVIEW,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "CITY_STATES_INTRO",
	Advisor = ADVISOR_FOREIGN,
	TitleText = "TXT_KEY_ADVISOR_CITY_STATES_INTRO_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_CITY_STATES_INTRO_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_CITY_STATE_GREETING,
	TutorialLevel = 1,
	Modal = true,		
};

Tutorial {
	ID = "TECH_TREE_SCREEN",
	Advisor = ADVISOR_SCIENCE,
	TitleText = "TXT_KEY_ADVISOR_SCREEN_TECH_TREE_DISPLAY",
	BodyText = "TXT_KEY_ADVISOR_SCREEN_TECH_TREE_BODY",
	ButtonPopupType = ButtonPopupTypes.BUTTONPOPUP_TECH_TREE,
	TutorialLevel = 1,
	Modal = true,		
};
