-------------------------------------------------
-- Select Map Type
-------------------------------------------------
include("IconSupport");
include("InstanceManager");

g_InstanceManager = InstanceManager:new( "ItemInstance", "Button", Controls.Stack );
local g_BackButton = {};
local g_isModding = false;

---------------------------------------------------------------- 
function IsWBMap(file)
	return Path.UsesExtension(file,".Civ5Map"); 
end

-- These are WB Maps that we explicitly do not want to display.
local worldBuilderMapsToFilter = {
	-- Filter out the earth maps since they are a special case.
	"Assets\\Maps\\Earth_Duel.Civ5Map",
	"Assets\\Maps\\Earth_Huge.Civ5Map",
	"Assets\\Maps\\Earth_Large.Civ5Map",
	"Assets\\Maps\\Earth_Small.Civ5Map",
	"Assets\\Maps\\Earth_Standard.Civ5Map",
	"Assets\\Maps\\Earth_Tiny.Civ5Map",
}


-------------------------------------------------
-------------------------------------------------
function OnBack()
    ContextPtr:SetHide( true );
    ContextPtr:LookUpControl( ".." ):SetHide( false );
end
Controls.CloseButton:RegisterCallback( Mouse.eLClick, OnBack );


----------------------------------------------------------------        
-- Input processing
----------------------------------------------------------------        
function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE then
            OnBack();
            return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );

function OnMapScriptSelected(script)
	PreGame.SetEarthMap(false);
	
	if(script == nil) then
		PreGame.SetRandomMapScript(true);
	else
		PreGame.SetRandomMapScript(false);
		
		local mapScript = "";
		if(type(script) == "string") then
			mapScript = script;
		else
			mapScript = script.FileName;
		end
		
		PreGame.SetMapScript(mapScript);
		
		if(IsWBMap(mapScript)) then
		
			local mapInfo = UI.GetMapPreview(mapScript);
			if(mapInfo ~= nil) then		
				local world = GameInfo.Worlds[mapInfo.MapSize];
			
				--Adjust Map Size
				PreGame.SetRandomWorldSize( false );
				PreGame.SetWorldSize( mapInfo.MapSize );
				PreGame.SetNumMinorCivs( world.DefaultMinorCivs );	
				
			end
		end
	end
	
	OnBack();
end

function OnEarthMapSelected()
	PreGame.SetEarthMap(true);
	OnBack();
end


function ShowHideHandler( bIsHide )
    if( not bIsHide ) then
		g_isModding = (ContextPtr:LookUpControl(".."):GetID() == "ModdingGameSetupScreen") or (ContextPtr:LookUpControl("../../.."):GetID() == "ModMultiplayerSelectScreen");
    
		if(g_isModding) then
			Controls.Title:SetText( Locale.ConvertTextKey("TXT_KEY_MAP_TITLE") );
		else
			Controls.Title:SetText( Locale.ConvertTextKey("TXT_KEY_MAP_TYPE_TITLE") );
		end
		
        Controls.ScrollPanel:SetScrollValue( 0 );
        
        RefreshMapScripts();
    end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

function RefreshMapScripts()

	g_InstanceManager:ResetInstances();
	
	function AddEarthMapEntry(entries)
		table.insert(entries, {
			Name = Locale.ConvertTextKey("TXT_KEY_MAP_EARTH_TITLE"),
			Description = Locale.ConvertTextKey("TXT_KEY_MAP_EARTH_HELP"),
			Callback = function() OnEarthMapSelected() end,
			Icon = {3, 64, "WORLDTYPE_ATLAS"}
		});
	end
	
	if(g_isModding) then
	
		local entries = {};
		AddEarthMapEntry(entries);
	
		-------------------------------------------------
		-- Populate list w/ Modded maps & regular & modded mapscripts
		-------------------------------------------------
		-- World builder Maps
		local maps = Modding.GetMapFiles();
		for _, map in ipairs(maps) do
		
			local bFilter = false;
			for i, mapFile in ipairs(worldBuilderMapsToFilter) do
				if(map.File == mapFile) then
					bFilter = true;
					break;
				end
			end
			
			if(not bFilter) then
				
				local mapData = UI.GetMapPreview(map.File);
				
				local name, description;
				local isError = nil;
				
				if(mapData) then
					if(not Locale.IsNilOrWhitespace(map.Name)) then
						name = map.Name;
					elseif(not Locale.IsNilOrWhitespace(mapData.Name)) then
						name = Locale.Lookup(mapData.Name);
					else
						name = Path.GetFileNameWithoutExtension(map.File);
					end
					
					if(not Locale.IsNilOrWhitespace(map.Description)) then
						description = map.Description;
					else
						description = Locale.Lookup(mapData.Description);
					end
					
					print(name);
				else
					name = Path.GetFileNameWithoutExtension(map.File);
					
					local nameTranslation = Locale.ConvertTextKey("TXT_KEY_INVALID_MAP_TITLE", name);
					if(nameTranslation and nameTranslation ~= "TXT_KEY_INVALID_MAP_TITLE") then
						name = nameTranslation;
					else
						name = "[COLOR_RED]" .. name .. "[ENDCOLOR]";
					end
					
					local descTranslation = Locale.ConvertTextKey("TXT_KEY_INVALID_MAP_DESC");
					if(translation and translation ~= "TXT_KEY_INVALID_MAP_DESC") then
						description = translation;
					else
						description = "[COLOR_RED]Invalid Map File[ENDCOLOR]";
					end
					
					isError = true;
				end
				
				local entryCallback = nil;
				if(not isError) then
					entryCallback = function() OnMapScriptSelected(map.File) end;
				end
				
				table.insert(entries, {
					Name = name,
					Description = description,
					Callback = entryCallback,
					Icon = {4, 64, "WORLDTYPE_ATLAS"},
				});				
			end
		end
		
		-- Modded Map Scripts
		-- Create a local table of map scripts first so that we can sort alphabetically
		for row in GameInfo.MapScripts() do
			table.insert(entries, {
				Name = Locale.ConvertTextKey(row.Name),
				Description = Locale.ConvertTextKey(row.Description or "");
				Icon = {row.IconIndex or 00, 64, row.IconAtlas},
				Callback = function() OnMapScriptSelected(row.FileName) end,
			});
		end	
		
		table.sort(entries, function(a, b) return a.Name < b.Name; end);
		
		for i, v in ipairs(entries) do
			local item = g_InstanceManager:GetInstance();
			IconHookup(v.Icon[1], v.Icon[2], v.Icon[3], item.Icon);
			item.Name:SetText(v.Name);
			item.Button:SetToolTipString(v.Description);
			item.Button:RegisterCallback(Mouse.eLClick, function() 
				if(v.Callback) then 
					v.Callback(); 
				end
			end);
		end
	else
		-------------------------------------------------
		-- Populate list w/ standard mapscripts
		-------------------------------------------------
		-- Add random map entry
		local item = g_InstanceManager:GetInstance();
		IconHookup( 4, 64, "WORLDTYPE_ATLAS", item.Icon );
		if(not PreGame.IsMultiplayerGame()) then
			item.Name:SetText( Locale.ConvertTextKey( "TXT_KEY_RANDOM_MAP_SCRIPT" ) );
			item.Button:SetToolTipString( Locale.ConvertTextKey( "TXT_KEY_RANDOM_MAP_SCRIPT_HELP" ) );
		else
			item.Name:SetText( Locale.ConvertTextKey( "TXT_KEY_ANY_MAP_SCRIPT" ) );
			item.Button:SetToolTipString( Locale.ConvertTextKey( "TXT_KEY_ANY_MAP_SCRIPT_HELP" ) );
		end
		item.Button:SetVoid1( -1 );
		item.Button:RegisterCallback( Mouse.eLClick, function() OnMapScriptSelected(); end);
		
		local entries = {};
		
		--Jon has specified that he wants only the following maps to be displayed and in the following order
		--	* Random Map (Random from the map set below)
		--	* Continents
		--	* Pangaea
		--  * Archipelago
		--  * Earth
		function AddMapScriptByFileName(fileName)
			for row in GameInfo.MapScripts{FileName = fileName, SupportsSinglePlayer = 1, Hidden = 0} do
				table.insert(entries, {
					Name = Locale.ConvertTextKey(row.Name),
					Description = Locale.ConvertTextKey(row.Description or ""),
					Icon = {row.IconIndex or 0, 64, row.IconAtlas},
					Callback = function() OnMapScriptSelected(row.FileName) end,
				});	
			end
		end
		
		AddMapScriptByFileName("Assets\\Maps\\Continents.lua");
		AddMapScriptByFileName("Assets\\Maps\\Pangaea.lua");
		AddMapScriptByFileName("Assets\\Maps\\Archipelago.lua");
		AddEarthMapEntry(entries);
		
		for i, v in ipairs(entries) do
			local item = g_InstanceManager:GetInstance();
			IconHookup(v.Icon[1], v.Icon[2], v.Icon[3], item.Icon);
			item.Name:SetText(v.Name);
			item.Button:SetToolTipString(v.Description);
			item.Button:RegisterCallback(Mouse.eLClick, v.Callback);
		end
	end
	

	Controls.Stack:CalculateSize();
	Controls.Stack:ReprocessAnchoring();
	Controls.ScrollPanel:CalculateInternalSize();
end

RefreshMapScripts();


