----------------------------------------------------------------        
-- Input Handler
----------------------------------------------------------------        
function InputHandler( uiMsg, wParam, lParam )

	if uiMsg == KeyEvents.KeyDown then

		if wParam == Keys.VK_ESCAPE then

			Matchmaking.LeaveMultiplayerGame();
			UIManager:DequeuePopup( ContextPtr );

		end

	end

	return true;

end
ContextPtr:SetInputHandler( InputHandler );


-------------------------------------------------
-- Event Handler: MultiplayerJoinRoomComplete
-------------------------------------------------
function OnJoinRoomComplete()

	if Matchmaking.IsHost() then

        UIManager:QueuePopup( Controls.StagingRoomScreen, PopupPriority.StagingScreen );
        UIManager:DequeuePopup( ContextPtr );
        
	else

		Controls.JoiningLabel:SetText( Locale.ConvertTextKey("TXT_KEY_MULTIPLAYER_JOINING_HOST" ));

	end

end


-------------------------------------------------
-- Event Handler: MultiplayerJoinRoomFailed
-------------------------------------------------
function OnJoinRoomFailed( iExtendedError, aExtendedErrorText )

	if iExtendedError == NetErrors.MISSING_REQUIRED_DATA then
		local szText = Locale.ConvertTextKey("TXT_KEY_MP_JOIN_FAILED_MISSING_RESOURCES");
		
		-- The aExtendedErrorText will contain an array of the IDs of the resources (DLC/Mods) needed by the game.
		local count = table.count(aExtendedErrorText);
		if(count > 0) then
			szText = szText .. "[NEWLINE]";
		    for index, value in pairs(aExtendedErrorText) do 		
				szText = szText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey(value);
			end
		end
		Events.FrontEndPopup.CallImmediate( szText );
	elseif iExtendedError == NetErrors.ROOM_FULL then
		Events.FrontEndPopup.CallImmediate( "TXT_KEY_MP_ROOM_FULL" );
	else
		Events.FrontEndPopup.CallImmediate( "TXT_KEY_MP_JOIN_FAILED" );
	end
	Matchmaking.LeaveMultiplayerGame();
	UIManager:DequeuePopup( ContextPtr );
	
end

-------------------------------------------------
-- Event Handler: MultiplayerConnectionFailed
-------------------------------------------------
function OnMultiplayerConnectionFailed()

	-- We should only get this if we couldn't complete the connection to the host of the room	
    Events.FrontEndPopup.CallImmediate( "TXT_KEY_MP_JOIN_FAILED" );
	Matchmaking.LeaveMultiplayerGame();
	UIManager:DequeuePopup( ContextPtr );
	
end

-------------------------------------------------
-- Event Handler: MultiplayerGameAbandoned
-------------------------------------------------
function OnMultiplayerGameAbandoned(eReason)

	if (eReason == NetKicked.BY_HOST) then
		Events.FrontEndPopup.CallImmediate( "TXT_KEY_MP_KICKED" );
	elseif (eReason == NetKicked.ROOM_FULL) then
		Events.FrontEndPopup.CallImmediate( "TXT_KEY_MP_ROOM_FULL" );
	else
	    Events.FrontEndPopup.CallImmediate( "TXT_KEY_MP_JOIN_FAILED" );
	end
	Matchmaking.LeaveMultiplayerGame();
	UIManager:DequeuePopup( ContextPtr );
	
end

-------------------------------------------------
-- Event Handler: ConnectedToNetworkHost
-------------------------------------------------
function OnConnect()

	if not Matchmaking.IsHost() then

		UIManager:QueuePopup( Controls.StagingRoomScreen, PopupPriority.StagingScreen );
        UIManager:DequeuePopup( ContextPtr );
	end
    
end


-------------------------------------------------
-- Show / Hide Handler
-------------------------------------------------
function ShowHideHandler( bIsHide, bIsInit )

    if( not bIsInit ) then
    	if not bIsHide then

			-- Activate only the DLC allowed for this MP game.  Mods will also deactivated/activate too.
			local prevCursor = UIManager:SetUICursor( 1 );
			Modding.ActivateAllowedDLC();
			UIManager:SetUICursor( prevCursor );
			
    		Controls.JoiningLabel:LocalizeAndSetText( "TXT_KEY_MULTIPLAYER_JOINING_ROOM" );
    		RegisterEvents();

    	else

    		UnregisterEvents();
    	end
	end

end
ContextPtr:SetShowHideHandler( ShowHideHandler );


-------------------------------------------------
-------------------------------------------------
function RegisterEvents()
    Events.MultiplayerJoinRoomComplete.Add( OnJoinRoomComplete );
    Events.MultiplayerJoinRoomFailed.Add( OnJoinRoomFailed );
    Events.MultiplayerConnectionFailed.Add( OnMultiplayerConnectionFailed );
    Events.MultiplayerGameAbandoned.Add( OnMultiplayerGameAbandoned );
    Events.ConnectedToNetworkHost.Add( OnConnect );
end


-------------------------------------------------
-------------------------------------------------
function UnregisterEvents()
    Events.MultiplayerJoinRoomComplete.Remove( OnJoinRoomComplete );
    Events.MultiplayerJoinRoomFailed.Remove( OnJoinRoomFailed );
    Events.MultiplayerConnectionFailed.Remove( OnMultiplayerConnectionFailed );
    Events.MultiplayerGameAbandoned.Remove( OnMultiplayerGameAbandoned );
    Events.ConnectedToNetworkHost.Remove( OnConnect );
end


-------------------------------------------------
-------------------------------------------------
function AdjustScreenSize()
    local _, screenY = UIManager:GetScreenSizeVal();
    local TOP_COMPENSATION = 52 + ((screenY - 768) * 0.3 );

    Controls.MainGrid:SetSizeY( screenY - TOP_COMPENSATION );
    Controls.MainGrid:ReprocessAnchoring();

end


-------------------------------------------------
-------------------------------------------------
function OnUpdateUI( type )
    if( type == SystemUpdateUIType.ScreenResize ) then
        AdjustScreenSize();
    end
end
Events.SystemUpdateUI.Add( OnUpdateUI );

AdjustScreenSize();
