-------------------------------------------------
-- Internet Lobby Screen
-------------------------------------------------
include( "SupportFunctions" );
include( "InstanceManager" );


-------------------------------------------------
-- Globals
-------------------------------------------------

local g_bSearching = false;

-- Hard coded DLC packages to ignore.
local DlcGuidsToIgnore = {
 "{8871E748-29A4-4910-8C57-8C99E32D0167}",
};

-- Listing Box Buttons
local g_InstanceManager = InstanceManager:new( "ListingButtonInstance", "Button", Controls.ListingStack );
local g_InstanceList = {};

-- Sort Option Data
-- Contains all possible buttons which alter the listings sort order.
g_SortOptions = {
	{
		Button = Controls.SortbyServer,
		Column = "ServerName",
		DefaultDirection = "asc",
		CurrentDirection = nil,
	},
	{
		Button = Controls.SortbyMapName,
		Column = "ServerMapName",
		DefaultDirection = "asc",
		CurrentDirection = nil,
	},
	{
		Button = Controls.SortbyMapSize,
		Column = "ServerWorldSize",
		DefaultDirection = "asc",
		CurrentDirection = nil,
	},
	{
		Button = Controls.SortbyDLCHosted,
		Column = "ServerHasDlc",
		DefaultDirection = "desc",
		CurrentDirection = nil,
	},
};

g_SortTable = {};

-------------------------------------------------
-- Server Listing Button Handler (Dynamic)
-------------------------------------------------
function ServerListingButtonClick( serverID )

	if serverID and serverID >= 0 then

		local bResult, bPending = Matchmaking.JoinMultiplayerGame( serverID );

	end

end


-------------------------------------------------
-- Host Game Button Handler
-------------------------------------------------
function HostButtonClick()

    UIManager:QueuePopup( Controls.MPGameSetupScreen, PopupPriority.MPGameSetupScreen );

end
Controls.HostButton:RegisterCallback( Mouse.eLClick, HostButtonClick );


-------------------------------------------------
-- Refresh Game List Button Handler
-------------------------------------------------
function RefreshButtonClick()

	if PreGame.IsInternetGame() then
		Matchmaking.RefreshInternetGameList(); -- Async
	else
		Matchmaking.RefreshLANGameList(); -- Async
	end

end
Controls.RefreshButton:RegisterCallback( Mouse.eLClick, RefreshButtonClick );


-------------------------------------------------
-- Back Button Handler
-------------------------------------------------
function BackButtonClick()

	HandleExitRequest();

end
Controls.BackButton:RegisterCallback( Mouse.eLClick, BackButtonClick );


----------------------------------------------------------------        
-- Input Handler
----------------------------------------------------------------        
function InputHandler( uiMsg, wParam, lParam )

	if uiMsg == KeyEvents.KeyDown then

		if wParam == Keys.VK_ESCAPE then

			HandleExitRequest();

		end

	end

	-- TODO: Is this needed?
	return true;

end
ContextPtr:SetInputHandler( InputHandler );


-------------------------------------------------
-- Screen Update Handler
-------------------------------------------------
--function OnUpdate()
--
	-- Debug TTY
--	if g_bSearching then
--		Controls.MultiplayerPanelDebug:SetText( Locale.ConvertTextKey("TXT_KEY_MULTIPLAYER_SEARCHING_GAME" ));
--	else
--		Controls.MultiplayerPanelDebug:SetText( Locale.ConvertTextKey("TXT_KEY_MULTIPLAYER_GAMES_FOUND").. " " .. Matchmaking.GetMultiplayerGameCount() );
--	end
--
--end
--ContextPtr:SetUpdate( OnUpdate );
--

-------------------------------------------------
-- Event Handler: MultiplayerGameListClear
-------------------------------------------------
function OnGameListClear()

	g_bSearching = true;

	UIManager:SetUICursor( 1 ); -- Busy

	Controls.RefreshButton:SetDisabled( true );
	Controls.HostButton:SetDisabled( true );

	UpdateGameList();

end
Events.MultiplayerGameListClear.Add( OnGameListClear );


-------------------------------------------------
-- Event Handler: MultiplayerGameListComplete
-------------------------------------------------
function OnGameListComplete()

	g_bSearching = false;

	UIManager:SetUICursor( 0 ); -- Normal

	Controls.RefreshButton:SetDisabled( false );
	Controls.HostButton:SetDisabled( false );

	UpdateGameList();

end
Events.MultiplayerGameListComplete.Add( OnGameListComplete );


-------------------------------------------------
-------------------------------------------------
function SelectGame( index )

	-- Reset the selection state of all the listings.
	if ( g_InstanceList ~= nil ) then
		for i,v in ipairs( g_InstanceList ) do -- Iterating over the entire list solves some issues with stale information.
			v.JoinButton:SetHide( true );
			v.SelectHighlight:SetHide( true );
		end
	end

    if ( index ~= nil and index ~= -1 and g_InstanceList[ index ] ~= nil ) then
		g_InstanceList[ index ].JoinButton:SetHide( false );
		g_InstanceList[ index ].SelectHighlight:SetHide( false );
    end

end


-------------------------------------------------
-------------------------------------------------
function UpdateGameList() 

	SelectGame(-1); -- Call this before clearing g_InstanceList to clear stale instance data.

	g_InstanceManager:ResetInstances();
	g_InstanceList = {};

	-- Get the Current Server List
	local serverTable = Matchmaking.GetMultiplayerGameList();
	
	local currentColor = 0; -- This toggles the color used for each row.
	
	g_SortTable = {};
	
	-- Display Each Server
	if serverTable then
		for i,v in ipairs( serverTable ) do
			local controlTable = g_InstanceManager:GetInstance();
			g_InstanceList[i] = controlTable;
			
			local serverEntry = serverTable[i];
			
			local sortData = {};
			g_SortTable[controlTable] = sortData;
						
			-- TODO: There must be a better way to do this.
			if currentColor == 1 then currentColor = 2 else currentColor = 1 end

			TruncateString(controlTable.ServerNameLabel, controlTable.ServerNameBox:GetSizeX(), serverEntry.serverName); 
			controlTable.ServerNameLabel:SetColorByName( "Beige_Black" );
			sortData.ServerName = serverEntry.serverName;

			controlTable.MembersLabel:SetText( serverEntry.numPlayers .. "/" .. serverEntry.maxPlayers );
			controlTable.MembersLabel:SetToolTipString(ParseServerPlayers(serverEntry.Players));
			controlTable.MembersLabel:SetColorByName( "Beige_Black" );
					
			-- Members column is sorted first by Max number of players, then by the number of free players
			local membersSortData = tostring(serverTable[i].maxPlayers);
			
			membersSortData = membersSortData .. tostring(serverEntry.maxPlayers - serverEntry.numPlayers);		
					
			-- Update Map Name
			do
				local label = controlTable.ServerMapTypeLabel;
				label:SetText("");
				label:SetToolTipString("");
				sortData.ServerMapName = "";
					
				local serverMapFile = serverTable[i].MapName;
				if(serverMapFile ~= nil and #serverMapFile > 0) then
							
					local bFound = false;			
					local serverMapFileName = Path.GetFileNameWithoutExtension(serverMapFile);
					
					-- First, attempt to discover the map name locally (for localization purposes).
					
					-- Check map scripts
					for row in GameInfo.MapScripts() do
						local mapFileName = Path.GetFileNameWithoutExtension(row.FileName);
						
						if(mapFileName == serverMapFileName) then
							local localizedMapName = Locale.ConvertTextKey(row.Name);
							label:SetText(localizedMapName);
							label:SetToolTipString(Locale.ConvertTextKey(row.Description or ""));
					
							sortData.ServerMapName = localizedMapName;				
							bFound = true;
							break;	
						end
					end
					
					-- Check WB maps
					if(not bFound) then

						local mapData = UI.GetMapPreview(serverMapFile);
						if(mapData) then
						
							local name, description;
								
							if(not Locale.IsNilOrWhitespace(mapData.Name)) then
								name = Locale.Lookup(mapData.Name);
							else
								name = Path.GetFileNameWithoutExtension(serverMapFile);
							end
							
							if(not Locale.IsNilOrWhitespace(mapData.Description)) then
								description = mapData.Description;
							else
								description = Locale.Lookup(mapData.Description);
							end
							
							label:SetText(name);
							label:SetToolTipString(description);
							
							sortData.ServerMapName = name;
						else
						
							-- Default to the filename.
							label:SetText(serverMapFileName);
							sortData.ServerMapName = serverMapFileName;
						end
					end
				end
			end
			
			-- Update World Size Field
			do
				local label = controlTable.ServerMapSizeLabel;
				local serverWorldSize = serverEntry.WorldSize;
				local serverWorldSizeTranslated = serverEntry.WorldSizeTranslated;
				
				if(serverWorldSize ~= nil and #serverWorldSize > 0) then
					if(Locale.HasTextKey(serverWorldSize)) then
						label:LocalizeAndSetText(serverWorldSize);
						sortData.ServerWorldSize = Locale.Lookup(serverWorldSize);
					else
						label:SetText(serverWorldSizeTranslated);
						sortData.ServerWorldSize = serverWorldSizeTranslated;
					end
				else
					label:SetText("");
					sortData.ServerWorldSize = "";
				end
			end
			
			local filteredDLC = serverEntry.hostedDLC;
			
			-- Filter out any packages in DlcToIgnore
			local packages = ParseDlcString(serverEntry.hostedDLC);
			local filteredPackages = {};
			for _, package in ipairs(packages) do
				local shouldFilter = false;
				for _, filter in ipairs(DlcGuidsToIgnore) do
					if(package == filter) then
						shouldFilter = true;
						break;
					end
				end 
				
				if(not shouldFilter) then
					table.insert(filteredPackages, package);
				end
			end
			
			if ( #filteredPackages > 0) then
				controlTable.DLCHostedLabel:LocalizeAndSetText( "TXT_KEY_MULTIPLAYER_LOBBY_YES" );
				controlTable.DLCHostedLabel:SetToolTipString(CreateDlcToolTip("TXT_KEY_LOBBY_REQUIRED_DLC", filteredPackages));
				sortData.ServerHasDlc = Locale.Lookup("TXT_KEY_MULTIPLAYER_LOBBY_YES");
				
				if ( serverTable[i].requiredDLCAvailable > 0 ) then
					controlTable.DLCHostedLabel:SetColorByName( "Beige_Black" );
				else
					controlTable.DLCHostedLabel:SetColorByName( "Gray_Black" );
				end
			else
				controlTable.DLCHostedLabel:SetToolTipString("");
				controlTable.DLCHostedLabel:LocalizeAndSetText( "TXT_KEY_MULTIPLAYER_LOBBY_NO" );
				sortData.ServerHasDlc = Locale.Lookup("TXT_KEY_MULTIPLAYER_LOBBY_NO");
				controlTable.DLCHostedLabel:SetColorByName( "Beige_Black" );
			end
					
			-- Enable the Button's Event Handler
			controlTable.Button:SetVoid1( i ); -- List ID
			controlTable.Button:RegisterCallback( Mouse.eLClick, SelectGame );
			
			-- Save the Server ID
			controlTable.JoinButton:SetVoid1( serverEntry.serverID ); -- Server ID
			controlTable.JoinButton:SetVoid2( 0 );
		
			-- Enable the Button's Event Handler
			controlTable.JoinButton:RegisterCallback( Mouse.eLClick, ServerListingButtonClick );
			controlTable.JoinButton:SetHide(true);

			-- Clear the Selection Highlight (It may have been set by a previous instance.)
			controlTable.SelectHighlight:SetHide( true );
		end
	end

	Controls.ListingStack:CalculateSize();
	Controls.ListingStack:ReprocessAnchoring();
	
	SortListings();

	Controls.ListingScrollPanel:CalculateInternalSize();

end

function ParseDlcString(dlcString)
	local packages = {};

	local guidLength = 38;
	local nPackages = math.floor(#dlcString / guidLength);
	
	for i = 1, nPackages, 1 do
		local index = (i-1) * guidLength;
		local packageId = string.sub(dlcString, index + 1, index + guidLength);
		table.insert(packages, packageId);
	end
	
	return packages;
end

function CreateDlcToolTip(headerString, packages)
	local lines = {Locale.Lookup(headerString)};
	
	for _, packageId in ipairs(packages) do
		
		local dlcName, dlcDescription = Modding.GetDlcNameDescriptionKeys(packageId);
		table.insert(lines, "[ICON_BULLET]" .. Locale.Lookup(dlcName));
	end
	
	return table.concat(lines, "[NEWLINE]");
end

function ParseServerPlayers(playerList)
	return string.gsub(playerList, ", ", "[NEWLINE]");
end


-------------------------------------------------
-- Callback for when sort options are selected.
-------------------------------------------------
function SortOptionSelected(option)
	--Current behavior is to only have 1 sort option enabled at a time 
	--though the rest of the structure is built to support multiple in the future.
	--If a sort option was selected that wasn't already selected, use the default 
	--direction.  Otherwise, toggle to the other direction.
	
	print("SORTING");
	for i,v in ipairs(g_SortOptions) do
		if(v == option) then
			if(v.CurrentDirection == nil) then			
				v.CurrentDirection = v.DefaultDirection;
			else
				if(v.CurrentDirection == "asc") then
					v.CurrentDirection = "desc";
				else
					v.CurrentDirection = "asc";
				end
			end
		else
			v.CurrentDirection = nil;
		end
	end
	
	SortListings();
end

-- Registers the sort option controls click events
function RegisterSortOptions()
	for i,v in ipairs(g_SortOptions) do
		if(v.Button ~= nil) then
			v.Button:RegisterCallback(Mouse.eLClick, function() SortOptionSelected(v); end);
		end
	end
end

function SortFunction(a, b)

	local aRowData = g_SortTable[ tostring( a ) ];
    local bRowData = g_SortTable[ tostring( b ) ];
    
    if(aRowData == nil) then
        return true;
    elseif(bRowData == nil) then
        return false;
    end
    
	for i,v in ipairs(g_SortOptions) do
		
		local compareResult = 0;
		if(v.CurrentDirection == "asc") then
			compareResult = Locale.Compare(aRowData[v.Column], bRowData[v.Column]);				
			
		elseif(v.CurrentDirection == "desc") then
			compareResult = Locale.Compare(bRowData[v.Column], aRowData[v.Column]);		
		end
		
		if(compareResult ~= 0) then
			return compareResult == -1
		end
    end
    
    return true;
end

function SortListings()
	Controls.ListingStack:SortChildren( SortFunction );
end

-------------------------------------------------
-- Leave the Lobby
-------------------------------------------------
function HandleExitRequest()

    UIManager:DequeuePopup( ContextPtr );

end


-------------------------------------------------
-------------------------------------------------
function ShowHideHandler( bIsHide )

	g_bSearching = false;
	g_InstanceManager:ResetInstances();
	g_InstanceList = {};

	if bIsHide then
		UIManager:SetUICursor( 0 ); -- Normal

	else	

		SelectGame(-1);
				
		local bIsModding = (ContextPtr:LookUpControl(".."):GetID() == "ModMultiplayerSelectScreen");
		if PreGame.IsInternetGame() then
			if(bIsModding) then
				Controls.TitleLabel:SetText( Locale.ConvertTextKey("TXT_KEY_MOD_INTERNET_LOBBY") );
			else
				Controls.TitleLabel:SetText( Locale.ConvertTextKey("TXT_KEY_MULTIPLAYER_INTERNET_LOBBY") );
			end
		else
			if(bIsModding) then
				Controls.TitleLabel:SetText( Locale.ConvertTextKey("TXT_KEY_MOD_LAN_LOBBY") );
			else
				Controls.TitleLabel:SetText( Locale.ConvertTextKey("TXT_KEY_MULTIPLAYER_LAN_LOBBY") );
			end
		end

	end

end
ContextPtr:SetShowHideHandler( ShowHideHandler )


-------------------------------------------------
-------------------------------------------------
function AdjustScreenSize()
    local _, screenY = UIManager:GetScreenSizeVal();
    
    local TOP_COMPENSATION = 52 + ((screenY - 768) * 0.3 );
    local BOTTOM_COMPENSATION = 192;
    local SIZE = screenY - (TOP_COMPENSATION + BOTTOM_COMPENSATION);

    Controls.ListingStack:SetSizeY( SIZE );
    Controls.ListingScrollPanel:SetSizeY( SIZE );
    Controls.ListingScrollPanel:CalculateInternalSize();
    Controls.MainGrid:SetSizeY( screenY - TOP_COMPENSATION );

end


-------------------------------------------------
-------------------------------------------------
function OnUpdateUI( type )
    if( type == SystemUpdateUIType.ScreenResize ) then
        AdjustScreenSize();
    end
end
Events.SystemUpdateUI.Add( OnUpdateUI );

AdjustScreenSize();


-------------------------------------------------
-- Event Handler: MultiplayerGameLaunched
-------------------------------------------------
function OnGameLaunched()

	UIManager:DequeuePopup( ContextPtr );

end
Events.MultiplayerGameLaunched.Add( OnGameLaunched );

-- Initialize screen
RegisterSortOptions();