-------------------------------------------------
-- Multiplayer Select Screen
-------------------------------------------------

local bIsModding = (ContextPtr:GetID() == "ModMultiplayerSelectScreen");

-------------------------------------------------
-- Internet Game Button Handler
-------------------------------------------------
function InternetButtonClick()

	PreGame.SetInternetGame( true );
	PreGame.SetGameType(GameTypes.GAME_NETWORK_MULTIPLAYER);
	PreGame.ResetSlots();
	Matchmaking.RefreshInternetGameList();
	UIManager:QueuePopup( Controls.LobbyScreen, PopupPriority.LobbyScreen );
	
end
Controls.InternetButton:RegisterCallback( Mouse.eLClick, InternetButtonClick );


-------------------------------------------------
-- LAN Game Button Handler
-------------------------------------------------
function LANButtonClick()

	PreGame.SetInternetGame( false );
	PreGame.SetGameType(GameTypes.GAME_NETWORK_MULTIPLAYER);
	PreGame.ResetSlots();
	Matchmaking.RefreshLANGameList();

	UIManager:QueuePopup( Controls.LobbyScreen, PopupPriority.LobbyScreen );

end
Controls.LANButton:RegisterCallback( Mouse.eLClick, LANButtonClick );

-------------------------------------------------
-- Hotseat Game Button Handler
-------------------------------------------------
function HotSeatButtonClick()

	PreGame.SetGameType(GameTypes.GAME_HOTSEAT_MULTIPLAYER);

	UIManager:QueuePopup( Controls.MPGameSetupScreen, PopupPriority.MPGameSetupScreen );

end
Controls.HotSeatButton:RegisterCallback( Mouse.eLClick, HotSeatButtonClick );

-------------------------------------------------
-- LAN Game Button Handler
-------------------------------------------------
function ReconnectButtonClick()
	Network.Reconnect();
	--PreGame.SetInternetGame( false );

	--Controls.LobbyScreen:SetHide( false );
end
Controls.ReconnectButton:RegisterCallback( Mouse.eLClick, ReconnectButtonClick );



-------------------------------------------------
-- Back Button Handler
-------------------------------------------------
function BackButtonClick()
	PreGame.SetInternetGame( false );
	PreGame.SetGameType( GAME_SINGLE_PLAYER );
	PreGame.ResetSlots();
    UIManager:DequeuePopup( ContextPtr );

end
Controls.BackButton:RegisterCallback( Mouse.eLClick, BackButtonClick );


-------------------------------------------------
-- Input Handler
-------------------------------------------------
function InputHandler( uiMsg, wParam, lParam )

	if uiMsg == KeyEvents.KeyDown then

		if wParam == Keys.VK_ESCAPE then

			BackButtonClick();

		end

	end

	return true;

end
ContextPtr:SetInputHandler( InputHandler );


-------------------------------------------------
-- Show / Hide Handler
-------------------------------------------------
function ShowHideHandler( bIsHide )

	if( not Network.IsConnectedToSteam() ) then
		Controls.InternetButton:SetDisabled( true );
		Controls.InternetButton:LocalizeAndSetToolTip("TXT_KEY_STEAM_CONNECTED_NO");
	end

	if not bIsHide then
		
		if(bIsModding) then
			Controls.Title:LocalizeAndSetText("{TXT_KEY_MODS_SELECT_MULTIPLAYER_TYPE:upper}");
		else
			Controls.Title:LocalizeAndSetText("{TXT_KEY_MULTIPLAYER_SELECT_MULTIPLAYER_TYPE:upper}");
		end
		
		if Network.HasReconnectCache() then
			Controls.ReconnectButton:SetHide( false);
			Controls.ReconnectButton:SetDisabled( false );
		else
			Controls.ReconnectButton:SetHide( true );
			Controls.ReconnectButton:SetDisabled( true );
			
			-- To prevent settings getting carried over from scenarios and what not
			-- reset pregame here.
			UIManager:SetUICursor( 1 );
	        Modding.ActivateDLC();
			PreGame.Reset();
			UIManager:SetUICursor( 0 );
		end
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

-------------------------------------------------
-- Event Handler: MultiplayerGameLaunched
-------------------------------------------------
function OnGameLaunched()

	UIManager:DequeuePopup( ContextPtr );

end
Events.MultiplayerGameLaunched.Add( OnGameLaunched );

