-------------------------------------------------
-- Premium Content Menu
-------------------------------------------------
include( "InstanceManager" );

g_InstanceManager = InstanceManager:new( "ListingButtonInstance", "Base", Controls.ListingStack );

g_DLCState = {};
--------------------------------------------------
-- Explicit Event Handlers
--------------------------------------------------
function InputHandler( uiMsg, wParam, lParam )

	if(uiMsg == KeyEvents.KeyDown) then

		if wParam == Keys.VK_ESCAPE then
			OnCancel();
		end
	end

	return true;
end
ContextPtr:SetInputHandler(InputHandler);

function OnCancel()
	UIManager:DequeuePopup(ContextPtr);
end
Controls.BackButton:RegisterCallback(Mouse.eLClick, OnCancel);

function OnOK()

	local packages = {}
	for i, v in ipairs(g_DLCState) do	
		if(v.Active ~= v.InitiallyActive) then
			table.insert(packages, {
				v.ID,
				ContentType.GAMEPLAY,
				v.Active
			});
		end
    end
    
    if(#packages > 0) then
		UIManager:SetUICursor(1);
		ContentManager.SetActive(packages);
		UIManager:SetUICursor(0);
    end
    
    UIManager:DequeuePopup(ContextPtr);
end
Controls.LargeButton:RegisterCallback(Mouse.eLClick, OnOK);

function RefreshDLC()

	g_InstanceManager:ResetInstances();
	
	local packageIDsToIgnore = {
	  	["8871E748-29A4-4910-8C57-8C99E32D0167"] = true, -- Upgrade 1
	};
	
	g_DLCState = {};
	local packageIDs = ContentManager.GetAllPackageIDs();
	for i, v in ipairs(packageIDs) do	
		
		if(packageIDsToIgnore[v] ~= true) then
		
			local bActive = ContentManager.IsActive(v, ContentType.GAMEPLAY);
			table.insert(g_DLCState, {
				ID = v,
				Description = Locale.Lookup(ContentManager.GetPackageDescription(v)),
				InitiallyActive = bActive,
				Active = bActive
			});			
		end
    end
    
    table.sort(g_DLCState, function(a,b)
		return Locale.Compare(a.Description, b.Description) == -1;
    end);
    
    local bHasExpansion = false;
        
    for i,v in ipairs(g_DLCState) do
    
		if(v.ID == "0E3751A1-F840-4E1B-9706-519BF484E59D") then
			bHasExpansion = true;
	
			Controls.ExpansionTitle:SetText(v.Description);
			Controls.EnableExpansion:SetHide(v.Active);
			Controls.DisableExpansion:SetHide(not v.Active);
			
			function ToggleExpansion()
				v.Active = not v.Active;
				Controls.EnableExpansion:SetHide(v.Active);
				Controls.DisableExpansion:SetHide(not v.Active);
			end
			
			Controls.EnableExpansion:RegisterCallback(Mouse.eLClick, ToggleExpansion);
			Controls.DisableExpansion:RegisterCallback(Mouse.eLClick, ToggleExpansion);
			
		else
			local controlTable = g_InstanceManager:GetInstance();
			controlTable.PackageName:SetText(v.Description);
			
			controlTable.EnableDLC:SetHide(v.Active);
			controlTable.DisableDLC:SetHide(not v.Active);
			
			function ToggleDLC()
				v.Active = not v.Active;
				controlTable.EnableDLC:SetHide(v.Active);
				controlTable.DisableDLC:SetHide(not v.Active);
			end
			controlTable.EnableDLC:RegisterCallback(Mouse.eLClick, ToggleDLC);
			controlTable.DisableDLC:RegisterCallback(Mouse.eLClick, ToggleDLC);
		end
    end
    
    Controls.ExpansionPanel:SetHide(not bHasExpansion);
        
	Controls.ListingStack:CalculateSize();
	Controls.ListingStack:ReprocessAnchoring();
	Controls.ListingScrollPanel:CalculateInternalSize();
end


function ShowHideHandler( bIsHide, bIsInit )
    if( not bIsHide ) then
		RefreshDLC();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

RefreshDLC();