-------------------------------------------------
-- Change player bumper screen
-------------------------------------------------
include( "IconSupport" );

local g_bIsOpen = false;
-------------------------------------------------
-------------------------------------------------
function OnContinue()
	if (PreGame.TestPassword( Game.GetActivePlayer(), Controls.EnterPasswordEditBox:GetText() )) then
		g_bIsOpen = false;
		UIManager:PopModal( ContextPtr );
	end
end
Controls.ContinueButton:RegisterCallback( Mouse.eLClick, OnContinue );

-------------------------------------------------
-------------------------------------------------
function OnChangePassword()
	UIManager:PushModal(Controls.ChangePassword, true);		
end
Controls.ChangePasswordButton:RegisterCallback( Mouse.eLClick, OnChangePassword );

---------------------------------------------------------------
function Validate()

	local bValid = PreGame.TestPassword( Game.GetActivePlayer(), Controls.EnterPasswordEditBox:GetText() );	
	Controls.ContinueButton:SetDisabled(not bValid);
end
---------------------------------------------------------------
Controls.EnterPasswordEditBox:RegisterCallback(Validate);

----------------------------------------------------------------        
----------------------------------------------------------------        
function OnSave()
    UIManager:PushModal( Controls.SaveMenu, false );
end
Controls.SaveButton:RegisterCallback( Mouse.eLClick, OnSave );

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function InputHandler( uiMsg, wParam, lParam )
	if (Game.GetNumGameTurnActive() ~= 0) then
		if uiMsg == KeyEvents.KeyDown then
			if wParam == Keys.VK_RETURN then
				OnContinue();
				return true;
			end
		end
	end
    return false;
end
ContextPtr:SetInputHandler( InputHandler );

-------------------------------------------------
-------------------------------------------------
function OnPasswordChanged( ePlayer )	
	local bShowPassword = PreGame.HasPassword( ePlayer );
	Controls.Stack:SetHide(not bShowPassword);
	
	-- Adjust the overall size of the box
	local iBoxHeight = 200;
	if ( bShowPassword ) then
		iBoxHeight = 268;
	end
	
	local frameSize = {};
	frameSize = Controls.MainBox:GetSize();
	frameSize.y = iBoxHeight;
	Controls.MainBox:SetSize( frameSize );
	frameSize.y = iBoxHeight;
	frameSize.y = iBoxHeight;
	
   	Controls.EnterPasswordEditBox:TakeFocus();
   	if (bShowPassword) then
   		local bValid = PreGame.TestPassword( Game.GetActivePlayer(), Controls.EnterPasswordEditBox:GetText());
		Controls.ContinueButton:SetDisabled( not bValid );   	
	else
		Controls.ContinueButton:SetDisabled( false );   	
	end
end

----------------------------------------------------------
----------------------------------------------------------
function UpdatePlayer()

	Controls.MainContainer:SetHide( false );
	local eActivePlayer = Game.GetActivePlayer();
	if (eActivePlayer ~= -1) then
		local szName = PreGame.GetNickName( eActivePlayer );		
		if (szName == "") then
			szName = PreGame.GetLeaderName( eActivePlayer );
			if (szName == "") then
				-- Get the player's leader type and get the leader description.
				local pPlayer = Players[ eActivePlayer ];
				local leaderType = pPlayer:GetLeaderType();			
				local leaderInfo = GameInfo.Leaders[leaderType];	
				szName = Locale.ConvertTextKey( leaderInfo.Description  );
			end			
		end
		Controls.Title:SetText( szName );
		
		local pPlayer = Players[eActivePlayer];
		if(pPlayer ~= nil) then
			local civ = GameInfo.Civilizations[pPlayer:GetCivilizationType()];
			if(civ ~= nil) then
				IconHookup( civ.PortraitIndex, 128, civ.IconAtlas, Controls.Icon );
			end
		end
		
	end
	Controls.EnterPasswordEditBox:SetText( "" );
	OnPasswordChanged( eActivePlayer );
end
-------------------------------------------------
-------------------------------------------------
function ShowHideHandler( bIsHide, bIsInit )
    
	if( not bIsHide ) then
		if (Game.GetNumGameTurnActive() == 0) then
			Controls.MainContainer:SetHide( true );
		else
			UpdatePlayer();
		end
	end	
	
    if( not bIsInit ) then
        if( not bIsHide ) then
			if (Game.GetNumGameTurnActive() ~= 0) then
				-- If this player is going to take a turn then pause
				Game.SetPausePlayer( Game.GetActivePlayer() );
			end
        	UI.incTurnTimerSemaphore();
        else
            UI.decTurnTimerSemaphore();
			Game.SetPausePlayer( -1 );
        end
        
   		g_bIsOpen = not bIsHide;
    end
	
end
ContextPtr:SetShowHideHandler( ShowHideHandler );


-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
LuaEvents.PasswordChanged.Add( function(ePlayer)
	OnPasswordChanged( ePlayer );
end);

-------------------------------------------------
-- Player changed handler
-------------------------------------------------
function OnPlayerChangedEvent()
	if (g_bIsOpen) then
		Game.SetPausePlayer( Game.GetActivePlayer() );
		UpdatePlayer();
	end
end
Events.GameplaySetActivePlayer.Add( OnPlayerChangedEvent );
