-------------------------------------------------
-- Options Menu
-------------------------------------------------
local m_FullscreenResList = {};
local m_iResolutionCount;
local m_maxX, m_maxY = OptionsManager.GetMaxResolution();

local m_bInGameQuickCombatState_Cached = false;
local m_bInGameQuickMovementState_Cached = false;

----------------------------------------------------------------
----------------------------------------------------------------
local m_WindowResList = {
{ x=2560, y=2048, bWide=false },
{ x=2560, y=1600, bWide=true },
{ x=1920, y=1200, bWide=true },
{ x=1920, y=1080, bWide=true },
{ x=1680, y=1050, bWide=true },
{ x=1600, y=1200, bWide=false },
{ x=1440, y=900,  bWide=true  },
{ x=1400, y=1050, bWide=true  },
{ x=1366, y=768,  bWide=true },
{ x=1280, y=1024, bWide=false },
{ x=1280, y=960,  bWide=true  },
{ x=1280, y=800,  bWide=true  },
{ x=1024, y=768,  bWide=false }, 
};

-- Store array of supported languages.
g_Languages = Locale.GetSupportedLanguages();
g_SpokenLanguages = Locale.GetSupportedSpokenLanguages();

-------------------------------------------------
-------------------------------------------------
local iMusicVolumeKnobID		= GetVolumeKnobIDFromName("USER_VOLUME_MUSIC");
local iSFXVolumeKnobID		= GetVolumeKnobIDFromName("USER_VOLUME_SFX");
local iSpeechVolumeKnobID		= GetVolumeKnobIDFromName("USER_VOLUME_SPEECH");
local iAmbienceVolumeKnobID	= GetVolumeKnobIDFromName("USER_VOLUME_AMBIENCE");

local m_cachedMusicVolumeKnob = nil;
local m_cachedSFXVolumeKnob = nil;
local m_cachedSpeechVolumeKnob = nil;
local m_cachedAmbienceVolumeKnob = nil;

function CacheAudioVolumes()
m_cachedMusicVolumeKnob = GetVolumeKnobValue(iMusicVolumeKnobID);
m_cachedSFXVolumeKnob = GetVolumeKnobValue(iSFXVolumeKnobID);
m_cachedSpeechVolumeKnob = GetVolumeKnobValue(iSpeechVolumeKnobID);
m_cachedAmbienceVolumeKnob = GetVolumeKnobValue(iAmbienceVolumeKnobID);
end

-------------------------------------------------
-------------------------------------------------
function RevertToCachedAudioVolumes()
	if( m_cachedMusicVolumeKnob ~= nil and m_cachedSFXVolumeKnob ~= nil and m_cachedSpeechVolumeKnob ~= nil 
	        and m_cachedAmbienceVolumeKnob ~= nil) then
		SetVolumeKnobValue(iMusicVolumeKnobID, m_cachedMusicVolumeKnob);
		SetVolumeKnobValue(iSFXVolumeKnobID, m_cachedSFXVolumeKnob);
		SetVolumeKnobValue(iSpeechVolumeKnobID, m_cachedSpeechVolumeKnob);
		SetVolumeKnobValue(iAmbienceVolumeKnobID, m_cachedAmbienceVolumeKnob);
	end
end

-------------------------------------------------
-------------------------------------------------
--function SetDefaultGameTabOptions()
--	OptionsManager.SetNoRewardPopups(false);
--	OptionsManager.SetNoTileRecommendations(false);
--	OptionsManager.SetCivilianYields(false);
--	OptionsManager.SetNoBasicHelp(false);
--	m_cachedSinglePlayerEndTurnTimerEnabled = false;
--	m_cachedMultiPlayerEndTurnTimerEnabled = true;
--	OptionsManager.SetTooltip1Seconds( 150 );
--	OptionsManager.SetTooltip2Seconds( 500 );
--	OptionsManager.SetTutorialLevel(0);
--end

-------------------------------------------------
-------------------------------------------------
function SetDefaultAudioVolumes()
	SetVolumeKnobValue(iMusicVolumeKnobID, 1);
	SetVolumeKnobValue(iSFXVolumeKnobID, 1);
	SetVolumeKnobValue(iSpeechVolumeKnobID, 1);
	SetVolumeKnobValue(iAmbienceVolumeKnobID, 1);
end

-------------------------------------------------
-------------------------------------------------
function OnOK()
	OptionsManager.CommitGameOptions();
	OptionsManager.CommitGraphicsOptions();
    SaveAudioOptions();
    
    local bIsInGame = (ContextPtr:GetID() == "OptionsMenu_InGame");
    
    if (bIsInGame) then
		local bCanCommit = false;
		if (PreGame.IsMultiplayerGame()) then
			if (Matchmaking.IsHost()) then
				-- If we are the host of a game, we can change the quick states
				bCanCommit = true;
			end
		else
			bCanCommit = true;
		end
		
		if (bCanCommit) then
			local options = {};
			if (m_bInGameQuickCombatState_Cached ~= PreGame.GetQuickCombat()) then
				table.insert(options, { "GAMEOPTION_QUICK_COMBAT", m_bInGameQuickCombatState_Cached });
			end
			
			if (m_bInGameQuickMovementState_Cached ~= PreGame.GetQuickMovement()) then
				table.insert(options, { "GAMEOPTION_QUICK_MOVEMENT", m_bInGameQuickMovementState_Cached });
			end
			
			Network.SendGameOptions(options);				
		end
	end
				
    --update local caches because the hide handler will set values back to cached versions
    CacheAudioVolumes();
    
    local bResolutionChanged = OptionsManager.HasUserChangedResolution();
    local bGraphicsChanged = OptionsManager.HasUserChangedGraphicsOptions();
    
    if( bResolutionChanged ) then
		OnApplyRes();
	end
	
	if( bGraphicsChanged ) then
		Controls.GraphicsChangedPopup:SetHide(false);
    end
    
    if( (not bGraphicsChanged) and (not bResolutionChanged) ) then
		OnBack();
	end
end
Controls.AcceptButton:RegisterCallback( Mouse.eLClick, OnOK );

-------------------------------------------------
-------------------------------------------------
function OnCancel()
	OptionsManager.SyncGameOptionsCache();
	OptionsManager.SyncGraphicsOptionsCache();
	OptionsManager.SyncResolutionOptionsCache();
	RevertToCachedAudioVolumes();
	OnBack();
end
Controls.CancelButton:RegisterCallback( Mouse.eLClick, OnCancel );

-------------------------------------------------
-------------------------------------------------
function OnDefault()
	OptionsManager.ResetDefaultGameOptions();
	OptionsManager.ResetDefaultGraphicsOptions();
	SetDefaultAudioVolumes();
	UpdateOptionsDisplay();
end
Controls.DefaultButton:RegisterCallback( Mouse.eLClick, OnDefault );

-------------------------------------------------
-------------------------------------------------
local fullscreenRes;
local windowedRes = {0,0};
local msaaSetting;
local bFullscreen;

function SavePreviousResolutionSettings()
	fullscreenRes = OptionsManager.GetResolution_Cached();
	windowedRes[1], windowedRes[2] = OptionsManager.GetWindowResolution_Cached();
	msaaSetting = OptionsManager.GetAASamples_Cached();
	bFullscreen = OptionsManager.GetFullscreen_Cached();
end

-------------------------------------------------
-------------------------------------------------
function RestorePreviousResolutionSettings()
	OptionsManager.SetResolution_Cached(fullscreenRes);
	OptionsManager.SetWindowResolution_Cached(windowedRes[1], windowedRes[2]);
	OptionsManager.SetAASamples_Cached(msaaSetting);
	OptionsManager.SetFullscreen_Cached(bFullscreen);
	if OptionsManager.HasUserChangedResolution() then
		OptionsManager.CommitResolutionOptions();
	end
	UpdateGraphicsOptionsDisplay();
end

-------------------------------------------------
-- If we hear a MultiplayerGameInvite was sent, exit
-- so we don't interfere with the transition
-------------------------------------------------
function OnMultiplayerGameInvite()
   	if( ContextPtr:IsHidden() == false ) then
		OnCancel();
	end
end

Events.MultiplayerGameInvite.Add( OnMultiplayerGameInvite );

-------------------------------------------------
-------------------------------------------------
local g_fTimer;
function OnUpdate( fDTime )
	Controls.CountdownTimer:SetText( Locale.ConvertTextKey("20") );
	
	g_fTimer = g_fTimer - fDTime;
	if( g_fTimer <= 0 ) then
	    ContextPtr:ClearUpdate();
        OnCountdownNo();
    else
    	Controls.CountdownTimer:SetText( string.format( "%i", g_fTimer + 1 ) );
	end
end
	
	
-------------------------------------------------
-------------------------------------------------
g_bIsResolutionCountdown = true;
function ShowResolutionCountdown()
	g_fTimer = 20;
	ContextPtr:SetUpdate( OnUpdate );
	Controls.Countdown:SetHide(false);
	Controls.CountdownMessage:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_RESOLUTION_TIMER") );
	Controls.CountYes:SetText( Locale.ConvertTextKey("TXT_KEY_YES_BUTTON") );
	Controls.CountNo:SetText( Locale.ConvertTextKey("TXT_KEY_NO_BUTTON") );
	Controls.CountdownTimer:SetText( "20" );
	Controls.LabelStack:CalculateSize();
	Controls.LabelStack:ReprocessAnchoring();
	g_bIsResolutionCountdown = true;
end

-------------------------------------------------
-------------------------------------------------
g_chosenLanguage = 0;
function ShowLanguageCountdown()
	g_fTimer = 20;
	ContextPtr:SetUpdate( OnUpdate );
	Controls.Countdown:SetHide(false);
	Controls.CountdownMessage:SetText( Locale.LookupLanguage(g_Languages[g_chosenLanguage].Type, "TXT_KEY_OPSCREEN_LANGUAGE_TIMER") );
	
	local YesText = string.format( "%s (%s)",
		Locale.LookupLanguage( g_Languages[g_chosenLanguage].Type, "TXT_KEY_YES_BUTTON"), 
		Locale.ConvertTextKey( "TXT_KEY_YES_BUTTON" ) );
	Controls.CountYes:SetText( YesText );
	
	local NoText = string.format( "%s (%s)",
		Locale.LookupLanguage( g_Languages[g_chosenLanguage].Type, "TXT_KEY_NO_BUTTON"), 
		Locale.ConvertTextKey( "TXT_KEY_NO_BUTTON" ) );
	Controls.CountNo:SetText( NoText );
	
	Controls.CountdownTimer:SetText( "20" );
	Controls.LabelStack:CalculateSize();
	Controls.LabelStack:ReprocessAnchoring();
	g_bIsResolutionCountdown = false;
end

-------------------------------------------------
-------------------------------------------------
function OnApplyRes()
	if( OptionsManager.HasUserChangedResolution() ) then
		ShowResolutionCountdown()
		OptionsManager.CommitResolutionOptions();
		UpdateGraphicsOptionsDisplay();
	end
end
Controls.ApplyResButton:RegisterCallback( Mouse.eLClick, OnApplyRes );

-------------------------------------------------
-------------------------------------------------
function OnResetTutorial()
	OptionsManager.ResetTutorial();
end
Controls.ResetTutorialButton:RegisterCallback( Mouse.eLClick, OnResetTutorial);

-------------------------------------------------
-------------------------------------------------
function OnOptionsEvent()
    UIManager:QueuePopup( ContextPtr, PopupPriority.OptionsMenu );
end

-------------------------------------------------
-------------------------------------------------
function ShowHideHandler( bIsHide, bIsInit )
    local bIsInGame = (ContextPtr:GetID() == "OptionsMenu_InGame"); --(UI.GetCurrentGameState() == GameStateTypes.CIV5_GS_MAINGAMEVIEW);
    
    if( bIsInit and bIsInGame ) then
        Events.EventOpenOptionsScreen.Add( OnOptionsEvent );
    end
    
    if( not bIsHide ) then
        --options menu is being shown
		OptionsManager.SyncGameOptionsCache();
		OptionsManager.SyncGraphicsOptionsCache();
		OptionsManager.SyncResolutionOptionsCache();
		
		if (bIsInGame) then 
			m_bInGameQuickCombatState_Cached = PreGame.GetQuickCombat();
			m_bInGameQuickMovementState_Cached = PreGame.GetQuickMovement();
		end
		
		CacheAudioVolumes();
		SavePreviousResolutionSettings();
        UpdateOptionsDisplay();
        
        if( bIsInGame and (PreGame.IsMultiplayerGame() or PreGame.IsHotSeatGame()) )then
            Controls.TutorialPull:SetDisabled( true );
            Controls.TutorialPull:SetAlpha( 0.5 );
            Controls.ResetTutorialButton:SetDisabled(true);
            Controls.ResetTutorialButton:SetAlpha( 0.5 );
        else
            Controls.TutorialPull:SetDisabled( false );
            Controls.TutorialPull:SetAlpha( 1.0 );
            Controls.ResetTutorialButton:SetDisabled(false);
            Controls.ResetTutorialButton:SetAlpha( 1.0 );
        end
        
        if( bIsInGame and PreGame.IsMultiplayerGame() and not Matchmaking.IsHost()) then
			-- Not this host, disable
			Controls.MPQuickCombatCheckbox:SetDisabled( true );
			Controls.MPQuickCombatCheckbox:SetAlpha( 0.5 );
			Controls.MPQuickMovementCheckbox:SetDisabled(true);
			Controls.MPQuickMovementCheckbox:SetAlpha( 0.5 );
		else
			Controls.MPQuickCombatCheckbox:SetDisabled( false );
			Controls.MPQuickCombatCheckbox:SetAlpha( 1.0 );
			Controls.MPQuickMovementCheckbox:SetDisabled(false);
			Controls.MPQuickMovementCheckbox:SetAlpha( 1.0 );
		end			        
        
        if( bIsInGame )then
            Controls.BGBlock:SetHide( false );
            Controls.VideoPanelBlock:SetHide( false );
            Controls.LanguagePull:SetDisabled( true );
            Controls.LanguagePull:SetAlpha( 0.5 );
            Controls.SpokenLanguagePull:SetDisabled(true);
            Controls.SpokenLanguagePull:SetAlpha( 0.5 );
            Controls.AutoUIAssetsCheck:SetDisabled( true );
            Controls.AutoUIAssetsCheck:SetAlpha( 0.5 );
            Controls.SmallUIAssetsCheck:SetDisabled( true );
            Controls.SmallUIAssetsCheck:SetAlpha( 0.5 );
        else
            Controls.VideoPanelBlock:SetHide( true );
            Controls.BGBlock:SetHide( true );
        end
    else
        --options menu is being hidden
	    RevertToCachedAudioVolumes();
	end
end
ContextPtr:SetShowHideHandler( ShowHideHandler );

-------------------------------------------------
-------------------------------------------------
function OnBack()
	if( Controls.GraphicsChangedPopup:IsHidden() and
		Controls.Countdown:IsHidden() ) then	
		UIManager:DequeuePopup( ContextPtr );
	end
	
	Controls.GraphicsChangedPopup:SetHide(true);
	OnCountdownNo();	
end
--Controls.CancelButton:RegisterCallback( Mouse.eLClick, OnBack );


-------------------------------------------------
-------------------------------------------------
function OnCategory( which )
    Controls.GamePanel:SetHide(  which ~= 1 );
    Controls.IFacePanel:SetHide( which ~= 2 );
    Controls.VideoPanel:SetHide( which ~= 3 );
    Controls.AudioPanel:SetHide( which ~= 4 );
    
    Controls.GameHighlight:SetHide(  which ~= 1 );
    Controls.IFaceHighlight:SetHide( which ~= 2 );
    Controls.VideoHighlight:SetHide( which ~= 3 );
    Controls.AudioHighlight:SetHide( which ~= 4 );
    
    Controls.TitleLabel:SetText( m_PanelNames[ which ] );
end
Controls.GameButton:RegisterCallback(  Mouse.eLClick, OnCategory );
Controls.IFaceButton:RegisterCallback( Mouse.eLClick, OnCategory );
Controls.VideoButton:RegisterCallback( Mouse.eLClick, OnCategory );
Controls.AudioButton:RegisterCallback( Mouse.eLClick, OnCategory );
Controls.GameButton:SetVoid1(  1 );
Controls.IFaceButton:SetVoid1( 2 );
Controls.VideoButton:SetVoid1( 3 );
Controls.AudioButton:SetVoid1( 4 );
m_PanelNames = {};
m_PanelNames[ 1 ] = Locale.ConvertTextKey( "TXT_KEY_GAME_OPTIONS" );
m_PanelNames[ 2 ] = Locale.ConvertTextKey( "TXT_KEY_INTERFACE_OPTIONS" );
m_PanelNames[ 3 ] = Locale.ConvertTextKey( "TXT_KEY_VIDEO_OPTIONS" );
m_PanelNames[ 4 ] = Locale.ConvertTextKey( "TXT_KEY_AUDIO_OPTIONS" );
OnCategory( 1 );


-------------------------------------------------
-- Volume control
-------------------------------------------------


function OnGameChangedVolumeLevel( iVolumeKnobID, fNewVolume )

	-- NOTE: testing controls by name is more hardcoded than we'd like
	-- it would be better to put the slider controls in an array and then test their void1's.
	
	-- NOTE: we may get messages for volume controls that are not user volume controls, just ignore them
	
	fRealVolume = GetVolumeKnobValue(iVolumeKnobID);
	
	-- format away any decimal junk by rounding down after padding a little bit.
	-- makes 0.8999 = 90%, 0.91 = 90%
	iNewVolume = math.floor((fNewVolume + 0.02) * 10) * 10;
	
	if Controls.MusicVolumeSlider:GetVoid1() == iVolumeKnobID then
		Controls.MusicVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MUSIC_SLIDER", iNewVolume ) );
	elseif Controls.EffectsVolumeSlider:GetVoid1() == iVolumeKnobID then
		Controls.EffectsVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SF_SLIDER", iNewVolume ) );
	elseif Controls.SpeechVolumeSlider:GetVoid1() == iVolumeKnobID then
		Controls.SpeechVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SPEECH_SLIDER", iNewVolume ) );
	elseif Controls.AmbienceVolumeSlider:GetVoid1() == iVolumeKnobID then
		Controls.AmbienceVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_AMBIANCE_SLIDER", iNewVolume ) );
	end
end
--Events.AudioVolumeChanged.Add( OnGameChangedVolumeLevel );

function OnUIVolumeSliderValueChanged( fNewVolumeLevel, void1 )
    SetVolumeKnobValue(void1, fNewVolumeLevel);
    
    ----copied from OnGameChangedVolumeLevel()----
	-- format away any decimal junk by rounding down after padding a little bit.
	-- makes 0.8999 = 90%, 0.91 = 90%
	local iNewVolume = math.floor((fNewVolumeLevel + 0.02) * 10) * 10;
	
	if Controls.MusicVolumeSlider:GetVoid1() == void1 then
		Controls.MusicVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MUSIC_SLIDER", iNewVolume ) );
	elseif Controls.EffectsVolumeSlider:GetVoid1() == void1 then
		Controls.EffectsVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SF_SLIDER", iNewVolume ) );
	elseif Controls.SpeechVolumeSlider:GetVoid1() == void1 then
		Controls.SpeechVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SPEECH_SLIDER", iNewVolume ) );
	elseif Controls.AmbienceVolumeSlider:GetVoid1() == void1 then
		Controls.AmbienceVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_AMBIANCE_SLIDER", iNewVolume ) );
	end
end

Controls.MusicVolumeSlider:RegisterSliderCallback(		OnUIVolumeSliderValueChanged );
Controls.EffectsVolumeSlider:RegisterSliderCallback(	OnUIVolumeSliderValueChanged );
Controls.SpeechVolumeSlider:RegisterSliderCallback(		OnUIVolumeSliderValueChanged );
Controls.AmbienceVolumeSlider:RegisterSliderCallback(	OnUIVolumeSliderValueChanged );

Controls.MusicVolumeSlider:SetVoid1(	iMusicVolumeKnobID );
Controls.EffectsVolumeSlider:SetVoid1(	iSFXVolumeKnobID );
Controls.SpeechVolumeSlider:SetVoid1(	iSpeechVolumeKnobID );
Controls.AmbienceVolumeSlider:SetVoid1(	iAmbienceVolumeKnobID );

Controls.MusicVolumeSlider:SetValue(	GetVolumeKnobValue(iMusicVolumeKnobID) );
Controls.EffectsVolumeSlider:SetValue(	GetVolumeKnobValue(iSFXVolumeKnobID) );
Controls.SpeechVolumeSlider:SetValue(	GetVolumeKnobValue(iSpeechVolumeKnobID) );
Controls.AmbienceVolumeSlider:SetValue( GetVolumeKnobValue(iAmbienceVolumeKnobID) );

Controls.MusicVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MUSIC_SLIDER", math.floor(GetVolumeKnobValue(iMusicVolumeKnobID)*100) ) );
Controls.EffectsVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SF_SLIDER", math.floor(GetVolumeKnobValue(iSFXVolumeKnobID)*100) ) );
Controls.SpeechVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SPEECH_SLIDER", math.floor(GetVolumeKnobValue(iSpeechVolumeKnobID)*100) ) );
Controls.AmbienceVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_AMBIANCE_SLIDER", math.floor(GetVolumeKnobValue(iAmbienceVolumeKnobID)*100) ) );

-------------------------------------------------
-------------------------------------------------
function Tooltip1TimerSliderChanged(value)
	
	--print("value: " .. value);
	
	i = math.floor(value * 100);
	i = i * 10;
	--print("i: " .. i);
	Controls.Tooltip1TimerLength:SetText(Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TOOLTIP_1_TIMER_LENGTH", math.floor(i / 100) ));
	OptionsManager.SetTooltip1Seconds_Cached(i);
end
Controls.Tooltip1TimerSlider:RegisterSliderCallback(Tooltip1TimerSliderChanged)
Controls.Tooltip1TimerSlider:SetValue(OptionsManager.GetTooltip1Seconds()/1000);


-------------------------------------------------
-------------------------------------------------
--[[
function Tooltip2TimerSliderChanged(value)
	i = math.floor(value * 100);
	i = i * 10;
	Controls.Tooltip2TimerLength:SetText(Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TOOLTIP_2_TIMER_LENGTH", i / 100));
	OptionsManager.SetTooltip2Seconds_Cached(i);
end
Controls.Tooltip2TimerSlider:RegisterSliderCallback(Tooltip2TimerSliderChanged)
Controls.Tooltip2TimerSlider:SetValue(OptionsManager.GetTooltip2Seconds()/1000);
--]]


-------------------------------------------------
-- Countdown handlers
-------------------------------------------------
function OnCountdownYes()
	if g_bIsResolutionCountdown == true then
		--just hide the menu
		Controls.Countdown:SetHide(true);
		SavePreviousResolutionSettings();
	else
		--apply language, reload UI
		Locale.SetCurrentLanguage( g_Languages[g_chosenLanguage].Type );
		Events.SystemUpdateUI( SystemUpdateUIType.ReloadUI );
	end
	--turn off timer
	ContextPtr:ClearUpdate();
end
Controls.CountYes:RegisterCallback( Mouse.eLClick, OnCountdownYes );

function OnCountdownNo()
	if g_bIsResolutionCountdown == true then
		--here we revert resolution options to some old options
		Controls.Countdown:SetHide(true);
		RestorePreviousResolutionSettings();
	else
		--revert language to current setting
		Controls.Countdown:SetHide(true);
		Controls.LanguagePull:GetButton():SetText(Locale.GetCurrentLanguage().DisplayName);
	end
	--turn off timer
	ContextPtr:ClearUpdate();
end
Controls.CountNo:RegisterCallback( Mouse.eLClick, OnCountdownNo );

function OnGraphicsChangedOK()
	--close the menu
	Controls.GraphicsChangedPopup:SetHide(true);
	OnBack();
end
Controls.GraphicsChangedOK:RegisterCallback( Mouse.eLClick, OnGraphicsChangedOK );

----------------------------------------------------------------
-- Input processing
----------------------------------------------------------------
function InputHandler( uiMsg, wParam, lParam )
    if uiMsg == KeyEvents.KeyDown then
        if wParam == Keys.VK_ESCAPE then
            OnBack();
            return true;
        end
    end
end
ContextPtr:SetInputHandler( InputHandler );

m_LeaderText = {};
m_LeaderText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MINIMUM");
m_LeaderText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_LeaderText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_LeaderText[ 3 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");

m_OverlayText = {};
m_OverlayText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_OverlayText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_OverlayText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");
    
m_ShadowText = {};
m_ShadowText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_OFF");
m_ShadowText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_ShadowText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_ShadowText[ 3 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");
   
m_FOWText = {};
m_FOWText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MINIMUM");
m_FOWText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_FOWText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_FOWText[ 3 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");
   
m_TerrainDetailText = {};
m_TerrainDetailText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MINIMUM");
m_TerrainDetailText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_TerrainDetailText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_TerrainDetailText[ 3 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");
  
m_TerrainTessText = {};
m_TerrainTessText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_TerrainTessText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_TerrainTessText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");

m_TerrainShadowText = {};
m_TerrainShadowText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_OFF");
m_TerrainShadowText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_TerrainShadowText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_TerrainShadowText[ 3 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");

m_WaterText = {};
m_WaterText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_WaterText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_MEDIUM");
m_WaterText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");

m_TextureQualityText = {};
m_TextureQualityText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_LOW");
m_TextureQualityText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SETTINGS_HIGH");

m_BindMouseText = {};
m_BindMouseText[ 0 ] = Locale.ConvertTextKey( "TXT_KEY_NEVER" );
m_BindMouseText[ 1 ] = Locale.ConvertTextKey( "TXT_KEY_FULLSCREEN_ONLY" );
m_BindMouseText[ 2 ] = Locale.ConvertTextKey( "TXT_KEY_ALWAYS" );

m_TutorialLevelText = {};
m_TutorialLevelText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TUTORIAL_LOW");
m_TutorialLevelText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TUTORIAL_MEDIUM");
m_TutorialLevelText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TUTORIAL_HIGH");
m_TutorialLevelText[ 3 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TUTORIAL_OFF");

m_MSAAText = {};
m_MSAAText[ 0 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MSAA_OFF");
m_MSAAText[ 1 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MSAA_2X");
m_MSAAText[ 2 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MSAA_4X");
m_MSAAText[ 3 ] = Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MSAA_8X");

m_MSAAMap = {};
m_MSAAMap[ 0 ] = 1;
m_MSAAMap[ 1 ] = 2;
m_MSAAMap[ 2 ] = 4;
m_MSAAMap[ 3 ] = 8;

m_MSAAInvMap = {};
m_MSAAInvMap[ 0 ] = 0;
m_MSAAInvMap[ 1 ] = 0;
m_MSAAInvMap[ 2 ] = 1;
m_MSAAInvMap[ 4 ] = 2;
m_MSAAInvMap[ 8 ] = 3;

----------------------------------------------------------------
-- Display updating
----------------------------------------------------------------

function UpdateGameOptionsDisplay()

	local bIsInGame = (ContextPtr:GetID() == "OptionsMenu_InGame");
	
    --Controls.NoCitizenWarningCheckbox:SetCheck( OptionsManager.IsNoCitizenWarning() );
    Controls.AutoWorkersDontReplaceCB:SetCheck( OptionsManager.IsAutoWorkersDontReplace_Cached() );
    Controls.AutoWorkersDontRemoveFeaturesCB:SetCheck( OptionsManager.IsAutoWorkersDontRemoveFeatures_Cached() );
    Controls.NoRewardPopupsCheckbox:SetCheck( OptionsManager.IsNoRewardPopups_Cached() );
    Controls.NoBasicHelpCheckbox:SetCheck( OptionsManager.IsNoBasicHelp_Cached() );
    Controls.NoTileRecommendationsCheckbox:SetCheck( OptionsManager.IsNoTileRecommendations_Cached() );
    Controls.CivilianYieldsCheckbox:SetCheck( OptionsManager.IsCivilianYields_Cached() );
    Controls.SinglePlayerAutoEndTurnCheckBox:SetCheck(OptionsManager.GetSinglePlayerAutoEndTurnEnabled_Cached());
	Controls.MultiplayerAutoEndTurnCheckbox:SetCheck(OptionsManager.GetMultiplayerAutoEndTurnEnabled_Cached());
	
	if (bIsInGame) then
		if (PreGame.IsMultiplayerGame() or PreGame.IsHotSeatGame()) then
			Controls.SPQuickCombatCheckBox:SetCheck(OptionsManager.GetSinglePlayerQuickCombatEnabled_Cached());
			Controls.SPQuickMovementCheckBox:SetCheck(OptionsManager.GetSinglePlayerQuickMovementEnabled_Cached());
			Controls.MPQuickCombatCheckbox:SetCheck(m_bInGameQuickCombatState_Cached);
			Controls.MPQuickMovementCheckbox:SetCheck(m_bInGameQuickMovementState_Cached);
		else
			Controls.SPQuickCombatCheckBox:SetCheck(m_bInGameQuickCombatState_Cached);
			Controls.SPQuickMovementCheckBox:SetCheck(m_bInGameQuickMovementState_Cached);
			Controls.MPQuickCombatCheckbox:SetCheck(OptionsManager.GetMultiplayerQuickCombatEnabled_Cached());
			Controls.MPQuickMovementCheckbox:SetCheck(OptionsManager.GetMultiplayerQuickMovementEnabled_Cached());
		end
	else
		Controls.SPQuickCombatCheckBox:SetCheck(OptionsManager.GetSinglePlayerQuickCombatEnabled_Cached());
		Controls.SPQuickMovementCheckBox:SetCheck(OptionsManager.GetSinglePlayerQuickMovementEnabled_Cached());
		Controls.MPQuickCombatCheckbox:SetCheck(OptionsManager.GetMultiplayerQuickCombatEnabled_Cached());
		Controls.MPQuickMovementCheckbox:SetCheck(OptionsManager.GetMultiplayerQuickMovementEnabled_Cached());
	end

	Controls.Tooltip1TimerLength:SetText(Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TOOLTIP_1_TIMER_LENGTH", math.floor(OptionsManager.GetTooltip1Seconds_Cached() / 100) ));
 	--Controls.Tooltip2TimerLength:SetText(Locale.ConvertTextKey("TXT_KEY_OPSCREEN_TOOLTIP_2_TIMER_LENGTH", OptionsManager.GetTooltip2Seconds_Cached() / 100));

    Controls.ZoomCheck:SetCheck( OptionsManager.GetStraightZoom_Cached() );
    Controls.PolicyInfo:SetCheck( OptionsManager.GetPolicyInfo_Cached() );
    Controls.AutoUnitCycleCheck:SetCheck( OptionsManager.GetAutoUnitCycle_Cached() );
    Controls.ScoreListCheck:SetCheck( OptionsManager.GetScoreList_Cached() );
    
    Controls.AutosaveTurnsEdit:SetText( OptionsManager.GetTurnsBetweenAutosave_Cached() );
    Controls.AutosaveMaxEdit:SetText( OptionsManager.GetNumAutosavesKept_Cached() );
    Controls.BindMousePull:GetButton():SetText( m_BindMouseText[ OptionsManager.GetBindMouseMode_Cached() ] );
	    
    local iTutorialLevel = OptionsManager.GetTutorialLevel_Cached();
	if (iTutorialLevel < 0) then
		iTutorialLevel = 3;
	end
	Controls.TutorialPull:GetButton():SetText( m_TutorialLevelText[ iTutorialLevel ] );
    
    Controls.LanguagePull:GetButton():SetText(Locale.GetCurrentLanguage().DisplayName);
    Controls.SpokenLanguagePull:GetButton():SetText(Locale.GetCurrentSpokenLanguage().DisplayName); --TODO: make this work like its friends -KS
	
	Controls.MusicVolumeSlider:SetValue(	GetVolumeKnobValue(iMusicVolumeKnobID) );
	Controls.EffectsVolumeSlider:SetValue(	GetVolumeKnobValue(iSFXVolumeKnobID) );
	Controls.SpeechVolumeSlider:SetValue(	GetVolumeKnobValue(iSpeechVolumeKnobID) );
	Controls.AmbienceVolumeSlider:SetValue( GetVolumeKnobValue(iAmbienceVolumeKnobID) );

	Controls.MusicVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_MUSIC_SLIDER", math.floor(GetVolumeKnobValue(iMusicVolumeKnobID)*100) ) );
	Controls.EffectsVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SF_SLIDER", math.floor(GetVolumeKnobValue(iSFXVolumeKnobID)*100) ) );
	Controls.SpeechVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_SPEECH_SLIDER", math.floor(GetVolumeKnobValue(iSpeechVolumeKnobID)*100) ) );
	Controls.AmbienceVolumeSliderValue:SetText( Locale.ConvertTextKey("TXT_KEY_OPSCREEN_AMBIANCE_SLIDER", math.floor(GetVolumeKnobValue(iAmbienceVolumeKnobID)*100) ) );

	
	if( OptionsManager.GetAutoUIAssets_Cached() ) then
        Controls.AutoUIAssetsCheck:SetCheck( true );
        Controls.SmallUIAssetsCheck:SetCheck( false );
        Controls.SmallUIAssetsCheck:SetDisabled( true );
        Controls.SmallUIAssetsCheck:SetAlpha( 0.5 );
    else
        Controls.AutoUIAssetsCheck:SetCheck( false );
        Controls.SmallUIAssetsCheck:SetDisabled( false );
        Controls.SmallUIAssetsCheck:SetAlpha( 1.0 );
        Controls.SmallUIAssetsCheck:SetCheck( OptionsManager.GetSmallUIAssets_Cached() );
    end
	
    Controls.SkipIntroVideoCheck:SetCheck( OptionsManager.GetSkipIntroVideo_Cached() );
	
	Controls.Tooltip1TimerSlider:SetValue(OptionsManager.GetTooltip1Seconds_Cached()/1000);
	--Controls.Tooltip2TimerSlider:SetValue(OptionsManager.GetTooltip2Seconds_Cached()/1000);
end
Events.GameOptionsChanged.Add(UpdateGameOptionsDisplay);


----------------------------------------------------------------
----------------------------------------------------------------
function UpdateGraphicsOptionsDisplay()
	--resolution options
	BuildFSResPulldown();
	BuildWResPulldown();
	
    local bIsFullscreen = OptionsManager.GetFullscreen_Cached(); 
    Controls.FullscreenCheck:SetCheck( bIsFullscreen );
    Controls.FSResolutionPull:SetHide( not bIsFullscreen);
    Controls.WResolutionPull:SetHide( bIsFullscreen );
    
    local kResInfo = m_FullscreenResList[ OptionsManager.GetResolution_Cached() ];
    if( kResInfo ~= nil ) then
        Controls.FSResolutionPull:GetButton():SetText( kResInfo.Width .. "x" .. kResInfo.Height .. "   " .. kResInfo.Refresh .. " Hz" );
	end
    	
    local x, y = OptionsManager.GetWindowResolution_Cached();
    Controls.WResolutionPull:GetButton():SetText( x .. "x" .. y );
    Controls.MSAAPull:GetButton():SetText( m_MSAAText[ m_MSAAInvMap[OptionsManager.GetAASamples_Cached()] ] );
    
    --graphics options
    Controls.VSyncCheck:SetCheck( OptionsManager.GetVSync_Cached() );
    Controls.HDStratCheck:SetCheck( OptionsManager.GetHDStrategicView_Cached() );
    Controls.LeaderPull:GetButton():SetText( m_LeaderText[ OptionsManager.GetLeaderQuality_Cached() ] );
	Controls.OverlayPull:GetButton():SetText( m_OverlayText[ OptionsManager.GetOverlayLevel_Cached() ] );
	Controls.ShadowPull:GetButton():SetText( m_ShadowText[ OptionsManager.GetShadowLevel_Cached() ] );
	Controls.FOWPull:GetButton():SetText( m_FOWText[ OptionsManager.GetFOWLevel_Cached() ] );
	Controls.TerrainDetailPull:GetButton():SetText( m_TerrainDetailText[ OptionsManager.GetTerrainDetailLevel_Cached() ] );
	Controls.TerrainTessPull:GetButton():SetText( m_TerrainTessText[ OptionsManager.GetTerrainTessLevel_Cached() ] );
	Controls.TerrainShadowPull:GetButton():SetText( m_TerrainShadowText[ OptionsManager.GetTerrainShadowQuality_Cached() ] );
	Controls.WaterPull:GetButton():SetText( m_WaterText[ OptionsManager.GetWaterQuality_Cached() ] );
	Controls.TextureQualityPull:GetButton():SetText( m_TextureQualityText[ OptionsManager.GetTextureQuality_Cached() ] );
end
Events.GraphicsOptionsChanged.Add(UpdateGraphicsOptionsDisplay);


----------------------------------------------------------------
----------------------------------------------------------------
function UpdateOptionsDisplay()
    UpdateGameOptionsDisplay();
    UpdateGraphicsOptionsDisplay();
end


----------------------------------------------------------------
-- Game Options Handlers
----------------------------------------------------------------
----------------------------------------------------------------
----------------------------------------------------------------
function OnNoCitizenWarningCheckbox( bIsChecked )
    OptionsManager.SetNoCitizenWarning_Cached( bIsChecked );
end
--Controls.NoCitizenWarningCheckbox:RegisterCheckHandler( OnNoCitizenWarningCheckbox );

----------------------------------------------------------------
----------------------------------------------------------------
function OnAutoWorkersDontReplaceCheckbox( bIsChecked )
    OptionsManager.SetAutoWorkersDontReplace_Cached( bIsChecked );
end
Controls.AutoWorkersDontReplaceCB:RegisterCheckHandler( OnAutoWorkersDontReplaceCheckbox );

----------------------------------------------------------------
----------------------------------------------------------------
function OnAutoWorkersDontRemoveFeaturesCheckbox( bIsChecked )
	OptionsManager.SetAutoWorkersDontRemoveFeatures_Cached( bIsChecked );
end
Controls.AutoWorkersDontRemoveFeaturesCB:RegisterCheckHandler( OnAutoWorkersDontRemoveFeaturesCheckbox );

----------------------------------------------------------------
----------------------------------------------------------------
function OnNoRewardPopupsCheckbox( bIsChecked )
    OptionsManager.SetNoRewardPopups_Cached( bIsChecked );
end
Controls.NoRewardPopupsCheckbox:RegisterCheckHandler( OnNoRewardPopupsCheckbox );

----------------------------------------------------------------
----------------------------------------------------------------
function OnNoTileRecommendationsCheckbox( bIsChecked )
    OptionsManager.SetNoTileRecommendations_Cached( bIsChecked );
end
Controls.NoTileRecommendationsCheckbox:RegisterCheckHandler( OnNoTileRecommendationsCheckbox );

----------------------------------------------------------------
----------------------------------------------------------------
function OnCivilianYieldsCheckbox( bIsChecked )
    OptionsManager.SetCivilianYields_Cached( bIsChecked );
end
Controls.CivilianYieldsCheckbox:RegisterCheckHandler( OnCivilianYieldsCheckbox );

----------------------------------------------------------------
----------------------------------------------------------------
function OnNoBasicHelpCheckbox( bIsChecked )
    OptionsManager.SetNoBasicHelp_Cached( bIsChecked );
end
Controls.NoBasicHelpCheckbox:RegisterCheckHandler( OnNoBasicHelpCheckbox );

----------------------------------------------------------------
----------------------------------------------------------------
function OnAutosaveTurnsChanged( string )
    OptionsManager.SetTurnsBetweenAutosave_Cached( string );
end
Controls.AutosaveTurnsEdit:RegisterCallback( OnAutosaveTurnsChanged );


----------------------------------------------------------------
----------------------------------------------------------------
function OnAutosaveMaxChanged( string )
    OptionsManager.SetNumAutosavesKept_Cached( string );
end
Controls.AutosaveMaxEdit:RegisterCallback( OnAutosaveMaxChanged );


----------------------------------------------------------------
----------------------------------------------------------------
function OnZoomCheck( bIsChecked )
    OptionsManager.SetStraightZoom_Cached( bIsChecked );
end
Controls.ZoomCheck:RegisterCheckHandler( OnZoomCheck );


----------------------------------------------------------------
----------------------------------------------------------------
function OnGridCheck( bIsChecked )
    OptionsManager.SetGridOn_Cached( bIsChecked );
end
--Controls.GridCheck:RegisterCheckHandler( OnGridCheck );

----------------------------------------------------------------
----------------------------------------------------------------
function OnSinglePlayerAutoEndTurnCheck( bIsChecked )
	OptionsManager.SetSinglePlayerAutoEndTurnEnabled_Cached(bIsChecked);
end
Controls.SinglePlayerAutoEndTurnCheckBox:RegisterCheckHandler(OnSinglePlayerAutoEndTurnCheck);

----------------------------------------------------------------
----------------------------------------------------------------
function OnMultiplayerAutoEndTurnCheck( bIsChecked )
	OptionsManager.SetMultiplayerAutoEndTurnEnabled_Cached(bIsChecked);
end
Controls.MultiplayerAutoEndTurnCheckbox:RegisterCheckHandler(OnMultiplayerAutoEndTurnCheck);

----------------------------------------------------------------
----------------------------------------------------------------
function OnSinglePlayerQuickCombatCheck( bIsChecked )
	local bIsInGame = (ContextPtr:GetID() == "OptionsMenu_InGame");
	if (bIsInGame) then
		if (not PreGame.IsMultiplayerGame() and not PreGame.IsHotSeatGame()) then
			-- In a single player game, store for updating the current game option
			m_bInGameQuickCombatState_Cached = bIsChecked;
		end
	end
	OptionsManager.SetSinglePlayerQuickCombatEnabled_Cached(bIsChecked);
end
Controls.SPQuickCombatCheckBox:RegisterCheckHandler(OnSinglePlayerQuickCombatCheck);

----------------------------------------------------------------
----------------------------------------------------------------
function OnSinglePlayerQuickMovementCheck( bIsChecked )
	local bIsInGame = (ContextPtr:GetID() == "OptionsMenu_InGame");
	if (bIsInGame) then
		if (not PreGame.IsMultiplayerGame() and not PreGame.IsHotSeatGame()) then
			-- In a single player game, store for updating the current game option
			m_bInGameQuickMovementState_Cached = bIsChecked;
		end
	end
	OptionsManager.SetSinglePlayerQuickMovementEnabled_Cached(bIsChecked);
end
Controls.SPQuickMovementCheckBox:RegisterCheckHandler(OnSinglePlayerQuickMovementCheck);

----------------------------------------------------------------
----------------------------------------------------------------
function OnMultiplayerQuickCombatCheck( bIsChecked )
	local bIsInGame = (ContextPtr:GetID() == "OptionsMenu_InGame");
	if (bIsInGame) then
		if (PreGame.IsMultiplayerGame() or PreGame.IsHotSeatGame()) then
			-- In a multiplayer game, store for updating the current game option
			m_bInGameQuickCombatState_Cached = bIsChecked;
		end
	end
	OptionsManager.SetMultiplayerQuickCombatEnabled_Cached(bIsChecked);
end
Controls.MPQuickCombatCheckbox:RegisterCheckHandler(OnMultiplayerQuickCombatCheck);

----------------------------------------------------------------
----------------------------------------------------------------
function OnMultiplayerQuickMovementCheck( bIsChecked )
	local bIsInGame = (ContextPtr:GetID() == "OptionsMenu_InGame");
	if (bIsInGame) then
		if (PreGame.IsMultiplayerGame() or PreGame.IsHotSeatGame()) then
			-- In a multiplayer game, store for updating the current game option
			m_bInGameQuickMovementState_Cached = bIsChecked;
		end
	end
	OptionsManager.SetMultiplayerQuickMovementEnabled_Cached(bIsChecked);
end
Controls.MPQuickMovementCheckbox:RegisterCheckHandler(OnMultiplayerQuickMovementCheck);

----------------------------------------------------------------
----------------------------------------------------------------
function OnAutoUI( bIsChecked )
    OptionsManager.SetAutoUIAssets_Cached( bIsChecked );
    --mini dispaly update here
    if( bIsChecked ) then
		Controls.SmallUIAssetsCheck:SetCheck( false );
        Controls.SmallUIAssetsCheck:SetDisabled( true );
        Controls.SmallUIAssetsCheck:SetAlpha( 0.5 );
    elseif( UI.GetCurrentGameState() == GameStateTypes.CIV5_GS_MAIN_MENU ) then
        Controls.SmallUIAssetsCheck:SetDisabled( false );
        Controls.SmallUIAssetsCheck:SetAlpha( 1.0 );
        Controls.SmallUIAssetsCheck:SetCheck( OptionsManager.GetSmallUIAssets_Cached() );
    end
end
Controls.AutoUIAssetsCheck:RegisterCheckHandler( OnAutoUI );


----------------------------------------------------------------
----------------------------------------------------------------
function OnSmallUI( bIsChecked )
    OptionsManager.SetSmallUIAssets_Cached( bIsChecked );
end
Controls.SmallUIAssetsCheck:RegisterCheckHandler( OnSmallUI );


----------------------------------------------------------------
----------------------------------------------------------------
function OnBindMousePull( level )
	OptionsManager.SetBindMouseMode_Cached( level );
    Controls.BindMousePull:GetButton():SetText( m_BindMouseText[ OptionsManager.GetBindMouseMode_Cached() ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnLanguagePull(level)
	level = level + 1; --offset because the pulldown is 0-based.
	Controls.LanguagePull:GetButton():SetText( g_Languages[level].DisplayName );
	g_chosenLanguage = level;
	ShowLanguageCountdown();
end

function OnSpokenLanguagePull(level) --TODO: hook this up too! -KS
	level = level + 1;
	Locale.SetCurrentSpokenLanguage( g_SpokenLanguages[level].Type );
	Controls.SpokenLanguagePull:GetButton():SetText( g_SpokenLanguages[level].DisplayName );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnPolicyInfo( bIsChecked )
    OptionsManager.SetPolicyInfo_Cached( bIsChecked );
end
Controls.PolicyInfo:RegisterCheckHandler( OnPolicyInfo );


----------------------------------------------------------------
----------------------------------------------------------------
function OnAutoUnitCycleCheck( bIsChecked )
    OptionsManager.SetAutoUnitCycle_Cached( bIsChecked );
end
Controls.AutoUnitCycleCheck:RegisterCheckHandler( OnAutoUnitCycleCheck );


----------------------------------------------------------------
----------------------------------------------------------------
function OnScoreListCheck( bIsChecked )
    OptionsManager.SetScoreList_Cached( bIsChecked );
end
Controls.ScoreListCheck:RegisterCheckHandler( OnScoreListCheck );

----------------------------------------------------------------
----------------------------------------------------------------
function OnSkipIntroVideo( bIsChecked )
    OptionsManager.SetSkipIntroVideo_Cached( bIsChecked );
end
Controls.SkipIntroVideoCheck:RegisterCheckHandler( OnSkipIntroVideo );

----------------------------------------------------------------
----------------------------------------------------------------
function OnTutorialPull( level )
    local iTutorialLevel = level;
	if (iTutorialLevel == 3) then
		iTutorialLevel = -1;
	end
    OptionsManager.SetTutorialLevel_Cached(iTutorialLevel);
	Controls.TutorialPull:GetButton():SetText( m_TutorialLevelText[ level ] );
end


----------------------------------------------------------------
-- Graphics Options Handlers
----------------------------------------------------------------

---------------------------------------------------------------
----------------------------------------------------------------
function OnHDStrategicView( bIsChecked )
    OptionsManager.SetHDStrategicView_Cached( bIsChecked );
end
Controls.HDStratCheck:RegisterCheckHandler( OnHDStrategicView );


----------------------------------------------------------------
----------------------------------------------------------------
function OnFullscreen( bIsChecked )
	OptionsManager.SetFullscreen_Cached( bIsChecked );
    Controls.FSResolutionPull:SetHide( not bIsChecked );
    Controls.WResolutionPull:SetHide( bIsChecked );
    
end
Controls.FullscreenCheck:RegisterCheckHandler( OnFullscreen );


----------------------------------------------------------------
----------------------------------------------------------------
function OnVSync( bIsChecked )
    OptionsManager.SetVSync_Cached( bIsChecked );
end
Controls.VSyncCheck:RegisterCheckHandler( OnVSync );

----------------------------------------------------------------
----------------------------------------------------------------
function OnLeaderPull( level )
    OptionsManager.SetLeaderQuality_Cached( level );
	Controls.LeaderPull:GetButton():SetText( m_LeaderText[ level ] );
end

----------------------------------------------------------------
----------------------------------------------------------------
function OnOverlayPull( level )
    OptionsManager.SetOverlayLevel_Cached( level );
	Controls.OverlayPull:GetButton():SetText( m_OverlayText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnShadowPull( level )
    OptionsManager.SetShadowLevel_Cached( level );
	Controls.ShadowPull:GetButton():SetText( m_ShadowText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnFOWPull( level )
    OptionsManager.SetFOWLevel_Cached( level );
	Controls.FOWPull:GetButton():SetText( m_FOWText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnTerrainDetailPull( level )
    OptionsManager.SetTerrainDetailLevel_Cached( level );
	Controls.TerrainDetailPull:GetButton():SetText( m_TerrainDetailText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnTerrainTessPull( level )
    OptionsManager.SetTerrainTessLevel_Cached( level );
	Controls.TerrainTessPull:GetButton():SetText( m_TerrainTessText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnTerrainShadowPull( level )
    OptionsManager.SetTerrainShadowQuality_Cached( level );
	Controls.TerrainShadowPull:GetButton():SetText( m_TerrainShadowText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnWaterPull( level )
    OptionsManager.SetWaterQuality_Cached( level );
	Controls.WaterPull:GetButton():SetText( m_WaterText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnTextureQualityPull( level )
    OptionsManager.SetTextureQuality_Cached( level );
	Controls.TextureQualityPull:GetButton():SetText( m_TextureQualityText[ level ] );
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnWResolutionPull( index )
    local kResInfo = m_WindowResList[ index ];

    if( kResInfo ~= nil ) then
        OptionsManager.SetWindowResolution_Cached( kResInfo.x, kResInfo.y );
    	Controls.WResolutionPull:GetButton():SetText( kResInfo.x .. "x" .. kResInfo.y );
	end

end
Controls.WResolutionPull:RegisterSelectionCallback( OnWResolutionPull );


----------------------------------------------------------------
----------------------------------------------------------------
function OnFSResolutionPull( index )
    local kResInfo = m_FullscreenResList[ index ];
    
    OptionsManager.SetResolution_Cached( index );
    Controls.FSResolutionPull:GetButton():SetText( kResInfo.Width .. "x" .. kResInfo.Height .. "   " .. kResInfo.Refresh .. " Hz" );
    
    OptionsManager.SetWindowResolution_Cached( kResInfo.Width, kResInfo.Height );
    Controls.WResolutionPull:GetButton():SetText( kResInfo.Width .. "x" .. kResInfo.Height );

end
Controls.FSResolutionPull:RegisterSelectionCallback( OnFSResolutionPull );

----------------------------------------------------------------
----------------------------------------------------------------
function OnMSAAPull( level )
	OptionsManager.SetAASamples_Cached( m_MSAAMap[level] );
	Controls.MSAAPull:GetButton():SetText( m_MSAAText[ level ] );
end
Controls.MSAAPull:RegisterSelectionCallback( OnMSAAPull );

----------------------------------------------------------------
-- Pulldowns
----------------------------------------------------------------

----------------------------------------------------------------
----------------------------------------------------------------
function BuildSinglePulldown( pulldown, textTable )
    local controlTable;
    for i = 0, #textTable do
        controlTable = {};
        pulldown:BuildEntry( "InstanceOne", controlTable );
        controlTable.Button:LocalizeAndSetText( textTable[i] );
        controlTable.Button:SetVoid1( i );
    end
    pulldown:CalculateInternals();
end


----------------------------------------------------------------
----------------------------------------------------------------
function BuildLeaderQualityPulldown()
    
    local controlTable;
    
    local size = #m_LeaderText;
    if UI.IsDX9() then
        size = size-1;        -- disallow high leaders on DX9, across the board
    end
    
    for i = 0, size do
    
        controlTable = {};
        Controls.LeaderPull:BuildEntry( "InstanceOne", controlTable );
        controlTable.Button:LocalizeAndSetText( m_LeaderText[i] );
        controlTable.Button:SetVoid1( i );
        
        -- disallow medium leaders on certain low-end machines
        if (i == 2 and not UI.AreMediumLeadersAllowed()) then
            controlTable.Button:SetDisabled(true);
            controlTable.Button:SetAlpha(.3);
        end
    
    end
    
    Controls.LeaderPull:CalculateInternals();
    
end

----------------------------------------------------------------
----------------------------------------------------------------
function BuildMSAAPulldown()
    local controlTable;
    for i = 0, #m_MSAAText do
        controlTable = {};
		Controls.MSAAPull:BuildEntry( "InstanceOne", controlTable );
		controlTable.Button:LocalizeAndSetText( m_MSAAText[i] );
		controlTable.Button:SetVoid1( i );
		if not OptionsManager.IsAALevelSupported(m_MSAAMap[i]) then
			controlTable.Button:SetDisabled(true);
			controlTable.Button:SetAlpha(.3);
		end
    end
    Controls.MSAAPull:CalculateInternals();
end

----------------------------------------------------------------
----------------------------------------------------------------
function BuildPulldowns()

    BuildFSResPulldown();
    BuildWResPulldown();
    BuildMSAAPulldown();
    
    BuildSinglePulldown( Controls.TutorialPull, m_TutorialLevelText );
    Controls.TutorialPull:RegisterSelectionCallback( OnTutorialPull );
    
    BuildSinglePulldown( Controls.OverlayPull, m_OverlayText );
    Controls.OverlayPull:RegisterSelectionCallback( OnOverlayPull );
    
    BuildSinglePulldown( Controls.ShadowPull, m_ShadowText );
    Controls.ShadowPull:RegisterSelectionCallback( OnShadowPull );

    BuildSinglePulldown( Controls.FOWPull, m_FOWText );
    Controls.FOWPull:RegisterSelectionCallback( OnFOWPull );
  
    BuildSinglePulldown( Controls.TerrainDetailPull, m_TerrainDetailText );
    Controls.TerrainDetailPull:RegisterSelectionCallback( OnTerrainDetailPull );
  
    BuildSinglePulldown( Controls.TerrainTessPull, m_TerrainTessText );
    Controls.TerrainTessPull:RegisterSelectionCallback( OnTerrainTessPull );

    BuildSinglePulldown( Controls.TerrainShadowPull, m_TerrainShadowText );
    Controls.TerrainShadowPull:RegisterSelectionCallback( OnTerrainShadowPull );

    BuildSinglePulldown( Controls.WaterPull, m_WaterText );
    Controls.WaterPull:RegisterSelectionCallback( OnWaterPull );

    BuildSinglePulldown( Controls.TextureQualityPull, m_TextureQualityText );
    Controls.TextureQualityPull:RegisterSelectionCallback( OnTextureQualityPull );


    ----------------------------------------------------------------
    -- leader detail pull (special case)
    local controlTable;
    local count = #m_LeaderText
    
    BuildLeaderQualityPulldown();
    Controls.LeaderPull:RegisterSelectionCallback( OnLeaderPull );
    ----------------------------------------------------------------


    BuildSinglePulldown( Controls.BindMousePull, m_BindMouseText );
    Controls.BindMousePull:RegisterSelectionCallback( OnBindMousePull );
    
    local languageTable = {};
    for i,v in ipairs(g_Languages) do
		languageTable[i - 1] = v.DisplayName;
    end
    BuildSinglePulldown( Controls.LanguagePull, languageTable);
    Controls.LanguagePull:RegisterSelectionCallback( OnLanguagePull );
    
    
    local spokenLanguageTable = {};
    for i,v in ipairs(g_SpokenLanguages) do
		spokenLanguageTable[i - 1] = v.DisplayName;
    end
    
    BuildSinglePulldown( Controls.SpokenLanguagePull, spokenLanguageTable);
    Controls.SpokenLanguagePull:RegisterSelectionCallback( OnSpokenLanguagePull );
end


----------------------------------------------------------------
----------------------------------------------------------------
function BuildWResPulldown()
    Controls.WResolutionPull:ClearEntries();
    m_maxX, m_maxY = OptionsManager.GetMaxResolution();

    local bFound = false;
    for i, kResInfo in ipairs( m_WindowResList ) do
        if( kResInfo.x <= m_maxX and
            kResInfo.y <= m_maxY ) then
            
            local controlTable = {};
            Controls.WResolutionPull:BuildEntry( "InstanceOne", controlTable );
            
            controlTable.Button:SetText( kResInfo.x .. "x" .. kResInfo.y );
            controlTable.Button:SetVoid1( i );
        end
    end
    
    Controls.WResolutionPull:CalculateInternals();
end


----------------------------------------------------------------
----------------------------------------------------------------
function BuildFSResPulldown()

    Controls.FSResolutionPull:ClearEntries();
    local count = #m_FullscreenResList;
    
    for i = count, 0, -1 do
        local kResInfo = m_FullscreenResList[i];
    
            --[[
            print( "Testing Mode: " .. kResInfo.Width .. " "
                                    .. kResInfo.Height .. " "
                                    .. kResInfo.Adapter .. " "
                                    .. kResInfo.Display .. " "
                                    .. kResInfo.Scale .. " "
                                    .. kResInfo.Refresh );--]]
                                   
        if( kResInfo.Height >= 768 and
            kResInfo.Adapter == 0 ) then
            
            local controlTable = {};
            Controls.FSResolutionPull:BuildEntry( "InstanceOne", controlTable );
            
            controlTable.Button:SetText( kResInfo.Width .. "x" .. kResInfo.Height .. "   " .. kResInfo.Refresh .. " Hz" );
            controlTable.Button:SetVoid1( i );
        end
    end
    
    Controls.FSResolutionPull:CalculateInternals();
end


----------------------------------------------------------------
-- build the internal list of fullscreen modes
----------------------------------------------------------------
m_iResolutionCount = UIManager:GetResCount();
for i = 0, m_iResolutionCount - 1 do
    local width, height, refresh, scale, display, adapter = UIManager:GetResInfo( i );
    m_FullscreenResList[i] = { Width = width,
                            Height = height,
                            Refresh = refresh,
                            Display = display,
                            Adapter = adapter };
end


----------------------------------------------------------------
-- add the machine's desktop resolution just in case they're
-- running something weird
----------------------------------------------------------------
local bFound = false;
for _, kResInfo in ipairs( m_WindowResList ) do
    if( kResInfo.x == m_maxX and
        kResInfo.y == m_maxY ) then
        bFound = true;
    end
end
if( bFound == false ) then
    m_WindowResList[0] = { x=m_maxX, y=m_maxY };
end


BuildPulldowns();
UpdateOptionsDisplay();
